#ifndef __ANALYSER_H__
#define __ANALYSER_H__

#include <analysable.h>
#include <utility/xmlelement.h>

#include <vector>


//! Ueberwachung der Messungen einer Simulationen.
/*!
 * Eine Instanz der Klasse Analyser verwaltet ein Liste von Komponenten,
 * die Messwerte fuer eine stochastische Auswertung erzeugen. Die Aufgabe 
 * eines Analysers ist es, die einzelnen Komponente aufzuforderen einen
 * Messwert fuer eine Messung bestimmten Typs zu erzeugen und diesen an
 * eine stochastischen Auswertung zu uebergeben. Da in der Regel mehrere
 * Messungen parallel ausgewertet werden, ist ein Analyser fuer die 
 * Aktualisierung einer Messung der stochastischen Auswertung verantwortlich.
 * Eine Messung wird dazu in Form einer Variablennummer spezifiziert, die 
 * bei stationaeren Simulationen (steady state) eindeutig ist. Im Falle von
 * transienten (terminating) Simulationen wird eine aufsteigende Folge 
 * von Variablen in Abhaengigkeit vom Takt der Messung benutzt, d.h. die
 * Messung, fuer die ein Analyser verantwortlich ist, umfasst dann 
 * eigenstaendige Messungen fuer jeden Takt der untersuchten transienten 
 * Phase.
 *
 * @see XMLElement
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Analyser : public std::vector<Analysable*>, public XMLElement
{
public:

	//! Konstruktor.
	/*!
	 * Die Klassenattribute werden mit Defaultwerten initialisiert.
	 * Zum Abschluss der Initialisierung ist Aufruf der Methode init()
	 * erforderlich.
	 * 
	 * @see init
	 * @param scope DOMElement mit Attributen fuer diesen Analyser.
	 */
    Analyser(const DOMElement* scope=NULL);
    
	//! Implementierung der abstrakten Klasse XMLElement.
	/*!
	 * Das DOMElement fuer diesen Analyser enthaelt Informationen zur Art 
	 * der zu ueberwachenden Messung und deren Parameter. Es werden weiterhin 
	 * die relevanten Variablen der stochastischen Auswertung in Abhaengigkeit
	 * von der Art der Simulation bestimmt.
	 * 
	 * @see XMLElement
	 */
	bool init(const DOMElement* scope=NULL);
	
	//! Messungen anstossen.
	/*!
	 * Ein Analyser wird einmal pro Takt ueber diese Methode aktiviert
	 * und veranlasst alle zugeordneten Komponenten ihre Messungen
	 * durchzufuehren.
	 */
    void observe();

    /*!
	 * Die Art der Messung.
	 * 
	 * @return Die Art der ueberwachten Messung.
	 */ 
	inline Analysable::TypeInfo ObserveType() const
	{
        return _observe;
    }

	/*! 
	 * Beschreibung der Messung.
	 * 
	 * @returns String mit kurzer Beschreibung der Messung.
	 */
    inline const std::string& Description() const
	{
        return _description;
    }

	/*! 
	 * Art der Auswertung.
	 * 
	 * @returns String mit kurzer Beschreibung der Messung.
	 */
    inline const std::string& Measurement() const
	{
        return _mtype;
    }

	/*!
	 * Die erste Variable der Messung.
	 * 
	 * @return Die erste Variable der Messung des Analysers.
	 */
    inline unsigned int FirstVariable() const
	{
        return _firstVariable;
    }
	
	/*!
	 * Die letzte Variable der Messung.
	 *
	 * @return Die letzte Variable der Messung des Analysers.
	 */
	inline unsigned int LastVariable() const
	{
		return _lastVariable;
	}

	
private:
       
	//! Art der Messung.
    Analysable::TypeInfo _observe;

	//! Kurze Beschreibung.
    std::string _description;

	//! Art der Auswertung.
	std::string _mtype;
	
	//! Zu ueberwachende Prioritaet.
    unsigned int _priority;

	//! Erste Variable fuer Messungen.
    unsigned int _firstVariable;

	//! Letzte Variable der Messung.
	unsigned int _lastVariable;
};

#endif
