//
// C++ Interface: arbiter.cpp
//
// Description:
//
//
// Author: Arvid Walter <asas@cs.tu-berlin.de>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef ARBITER_H
#define ARBITER_H

#include <utility/xmlelement.h> // Arbiter sind XMLElemente
#include <vector> // scheduleListe

#include <wire.h> // scheduler, scheduleListe etc.
#include <utility/factory.h> // Arbiter sind Factory-Elemente
#include <signature.h> // Arbiter koennen signiert werden

#include <queue>

//fwd. declaration fuer den fried zugriff
class ArbiterControler;

/*! Der Arbiter.
 *  Ein Arbiter soll fr einen gegebenen Ausgang aus konkurrierenden Eingngen den Eingang ermitteln, 
 *  der in diesem Takt auf den Ausgang geschaltet werden soll. 
 *  Der Arbiter bestimmt dazu aus der fr den Ausgang relevanten Zeile der 
 *  Schalt-Matrix des Routers die konkurrienden Eingnge, wobei davaon ausgegangen wird, 
 *  dass diese auch wirklich den Ausgang benutzen wollen bzw. knnen. 
 *  Bedingungen wie z.B. Backpressure werden vom Arbiter nicht geprft.
 *  @author Arvid Walter
 */
class Arbiter : public XMLElement, public FactoryElement, public Signature
{
	friend class Test_LFUArbiter;
	friend class Test_LRUArbiter;
	friend class Test_MFUArbiter;
	friend class Test_MRUArbiter;
	friend class Test_OPFArbiter;
	friend class Test_WPFArbiter;
	friend class Test_DeadlineArbiter;
	
public:
        //! Der Arbiter-Konstruktor
        /*!
         * Konstruiert einen Arbiter, einen Scheduler.
         * @arg scope Ein Zeiger auf das Konfigurationselement des Arbiters.
         */
        Arbiter(const DOMElement* scope=NULL); // Konstruktor

        virtual ~Arbiter() // virtueller Destruktor
        {}
        ;

        //! Arbiter-Initialisierung
        /*!
         * Initialisiert den Arbiter. Wird kein Argument bergeben,
         * sollte bereits eines fr den Arbiter in seiner Eigeneschaft als
         * XMLElement gesetzt worden sein.
         * @return false Wenn keine Initialisierung vorgenommen werden konnte
         * @see XMLElement::setDomElement()
         * @param scope Ein Zeiger auf das Konfigurationselement des Arbiters.
         */

        virtual bool init(const DOMElement* scope=NULL) = 0; // XML-parsen und den Arbiter konfigurieren

        //! Uebergibt dem Arbiter alle moeglichen Eingaenge
        virtual void setInputs(const std::vector<Wire*>*);

        //! Gibt das erste Element der zugeteilten Elemente wieder oder NULL
        /*! PRE: schedule ist aufgerufen worden */
        virtual Wire* pop() = 0;

	//! Gibt Gewinner wieder, Ergebnis bleibt unveraendert.
	/*! PRE: schedule ist aufgerufen worden */
	virtual Wire* front() const = 0;
	
	void setIndex(unsigned int index) { a_index = index; }

	unsigned int Index() const { return a_index; }
	
        void Schedule(std::vector<Wire*> ins);
	void Forget();
	void Accept(const std::vector<Wire*>& accepted_list);
	
protected:

        //! Gibt die Eingaenge des Schedulers zurueck
        const std::vector<Wire*>* Inputs();

private:
        //! Fhrt einen Reset bzgl. des Taktes des arbiters aus
        /*! Hier sollten interne Listen zur Verwaltung des Schedulers zurckgesetzt werden
         *  Ein reset bezieht sich hier auf den Takt - nicht auf die Simulation
         */
        virtual const void forget() = 0;
        
	//! Ein Handshake zwischen ArbiterControler und Arbiter.
        /*! Der ArbiterControler teilt dem Arbiter mit, wie seine entscheidung
         * bzgl. des Vorschlages des Arbiters ausgefallen ist.
         * Der Arbiter kann mit diesen Informationen fr das ihm zu Grunde liegden Verfahren notwendige
         * Paramter neu ermitteln.
         * Die hier uebergebenen Werte drfen sich nicht mehr in diesem Takt verndern !
         * @param accepted_list Der tatschliche Zustand der Zuteilung am Ende des Taktes.
         */
        virtual const void accept(const std::vector<Wire*>& accepted_list) = 0; // Die Scheduleliste, wie sie von AC angenommen worden ist.
	
	//! Die Menge der Eingnge wird nach dem entspr. impl. Verfahren sortiert
        /*! Hier wird von jedem einzelnen Arbiter ein eigenenes Scheduleverfahren implementiert.
         * @param ins Die zuzuteilenden Eingnge
         */
        virtual void schedule(std::vector<Wire*> ins) = 0; // Konflikte Loesen anhand eines entspr. Verfahrens
        
	const std::vector<Wire*>* inputs;
	
	struct state {
		bool forgot;
		bool accepted;
	} my_state;

	unsigned int a_index;
};

#endif
