#ifndef ARBITERCONTROLLER_H
#define ARBITERCONTROLLER_H

#include <utility/xmlelement.h>
#include <utility/factory.h>
#include <wirematrixmodifier.h>

#include <vector>

class Arbiter;
class Wire;
class WireMatrix;
class WireMatrixEntry;

//! Arbiter-Controller eines Routers.
/*!
 * Diese abstrakte Klasse beschreibt einen Controller fuer die Arbiter an
 * den Ausgaengen eines Routers. Ein Arbiter implementiert ein Scheduling-
 * Verfahren, mit dem aus relavanten Eingaengen ein Eingang ausgewaehlt 
 * wird, der auf den Ausgang des Arbiters geschaltet werden soll. Diese 
 * Entscheidung erfolgt lokal, d.h. nur unter Einbeziehung der Eingaenge 
 * und Informationen zum Ausgang (z.B. letzte Entscheidung). Eine Instanz 
 * dieser Klasse verwaltet alle Arbiter der Ausgaenge eines Routers, bestimmt 
 * bei Bedarf die relevanten Eingaenge zu einem Ausgang und aktiviert
 * den entsprechenden Arbiter. Die getroffene Entscheidung kann akzeptiert
 * oder aber zurueckgestellt werden, um optimales Redundanzverhalten zu
 * realisieren. Die Entscheidung eines Arbiters wird durch die Implementierung
 * der Klasse WireMatrixModifier auf die Schaltmatrix des zugehoerigen
 * Routers uebertragen.
 *
 * @see WireMatrixModifier
 * @see Arbiter
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ArbiterController : public XMLElement, public FactoryElement, public WireMatrixModifier
{
public:
        //! Standard-Konstruktor
        /*! 
	 * Eine initiale Instanz dieser Klasse besitzt keine direkte 
	 * Verbindung zur Schaltmatrix des zueghoerigen Routers und 
	 * damit zu den Ausgaengen und Leitungen. Die einzelnen Arbiter 
	 * mssen explizit hinzugefuegt werden.
	 * 
         * @see addArbiter
	 * 
         * @param dom Der zugehoerige Abschnitt der XML-Konfiguration.
         */
        ArbiterController(const DOMElement* dom=NULL) : _wm(NULL) {}

	//! Destruktor.
	/*! 
	 * Der Aufruf des Destruktors sorgt dafuer, dass die verwalteten 
	 * Arbiter zerstrt werden.
	 */
        virtual ~ArbiterController();

        //! Implementierung der abstrakten Basisklasse XMLElement.
        bool init(const DOMElement* scope=NULL);
	
	//! Implementierung der abstrakten Basisklasse WireMatrixModifer.
	/*!
	 * Bevor an der Schaltmatrix des zugehoerigen Routers Modifikationen
	 * fuer einen Ausgang vorgenommen werden, muss sichergestellt werden,
	 * dass das Scheduling-Ergebnis auch aktuell ist. Das ist genau dann 
	 * der Fall, wenn der enstprechende Arbiter fuer den aktuellen Takt
	 * signiert ist.
	 *
	 * @see WireMatrixModifier
	 */
	int precondition(WireMatrixIterator* it);
	
	//! Implementierung der abstrakten Basisklasse WireMatrixModifier.
	/*!
	 * Diese Methode sorgt dafuer, dass eine getroffene Scheduling-
	 * Entscheidung auf die Schaltmatrix des zugehoerigen Routers
	 * uebertragen wird.
	 *
	 * @see WireMatrixModifier
	 */
	void modify(WireMatrixIterator* it);

        //! Hinzufuegen eines Arbiters.
	/*!
	 * Diese Methode fuegt einen zu verwaltenden Arbiter zur Arbiter-Liste
	 * dieses Controllers hinzu. Der Aufruf erfolgt fuer jeden Ausgang der
	 * zugehoerigen Schaltmatrix. Der Index des eines Arbiters entspricht
	 * damit dem (Ausgangs-)Index eines Ausgangs des zugehoerigen Routers.
	 *
	 * @return true, falls Instanziierung des Arbiters erfolgreich.
	 */
        bool addArbiter();
	
	//! Zugriff auf Arbiter einer Leitung.
	/*!
	 * Diese Methode liefert den verwalteten Arbiter zum uebergebenen
	 * Ausgang. Es findet keine Ueberpruefung auf korrekt spezifizierten 
	 * Ausgang statt.
	 * 
	 * @param outwire Ein Ausgang des zugehoerigen Routers.
	 * @return Arbiter zu uebergebenen Ausgang.
	 */
	const Arbiter* getArbiter(Wire* outwire) const; 

        //! Setzen der Schaltmatrix des zugehoerigen Routers.
        /*! 
         * Zur Realisierung komplexen Redundanzverhaltens und Uebertragung von
	 * Scheduling-Entscheidungen ist der Zugriff auf die Schaltmatrix des 
	 * zugehoerigen Routers erforderlich.
	 * 
	 * @param wm Die Schaltmatrix des Routers zu diesem ArbiterController.
         */
        void setMatrix(WireMatrix* wm){ _wm = wm; }
		
	//! Test auf Kandidat fuer Ausgang.
	/*!
	 * Diese Methode prueft fuer einen Eintrag der Matrix, ob sich
	 * der beteiligte Eingang, um den Ausgang bewirbt und am
	 * Scheduling teilnehmen kann. Ein Eingang gilt als Bewerber,
	 * falls Ein-/Ausgang noch ungeschaltet und aktiv ist und ein Paket
	 * anliegt, dass den Ausgang benutzen kann und dort Platz findet.
	 *
	 * @param entry Eintrag der Matrix.
	 * @return true, falls Eingang zulaessiger Bewerber fuer Ausgang.
	 */
	bool competitor(WireMatrixEntry& entry) const;
	
	//! Aktivierung des Arbiter-Contollers.
	/*! 
	 * Der Aufruf dieser Methode aktiviert den Arbiter-Controller mit
	 * dem uebergebenen Ausgang als Startpunkt. Das Ziel sollt nun sein,
	 * aus allen relevanten Eingaengen einen passenden Eingang zu
	 * bestimmen, der auf den uebergebenen Ausgang geschaltet wird. Dabei
	 * wird der Arbiter des spezifierten Ausganges verwendet. Bei komplexen
	 * Redundanzverhalten koennen auch weitere Arbiter hinzugenommen und
	 * damit weitere Ausgaenge bearbeitet werden.
	 * 
	 *  @param output Ein zu vermittelnder Ausgang
	 *  @return true Bei Vermittlung, false bei verschobener Entscheidung.
	 */
	virtual bool activate(Wire* output) = 0;
	
protected:
	
	//! Die Liste der Arbiter (@see addArbiter).
        std::vector<Arbiter*> arbiter;
	
	//! Die Schaltmatrix des zugehoerigen Routers.
        WireMatrix *_wm;
};

#endif
