#ifndef __BUFFERQUEUE_H__
#define __BUFFERQUEUE_H__

#include <list>

class Flit;
class Buffer;
class RegisterContainer;

#include <signature.h>

//! Die interne Wartschlange eines Puffers.
/*!
 * Die internen Warteschlangen (Queues) der Puffer verwalten Paketfragmente, 
 * die Flits, auf ihrem Weg durch das Netz. Ein Flit kann eine BufferQueue
 * verlassen, indem es in das Sende-Register verschoben und dieses mit einer
 * (ausgehenden) Leitung verbunden wird.
 *
 * @see RegisterContainer
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class BufferQueue 
{
// friends duerfen Queue aendern, alle anderen nur lesen
friend class Test_RegisterContainer;
friend class Buffer;
friend class Simulation;
friend class SourceBuffer;
friend class DLHandler;

public:
	//! Standard-Konstruktor.
	/*!
	 * Der Aufruf des Konstruktors erzeugt mindestens eine Queue der Groesse1.
	 *
	 * @param space Anzahl der verwaltbaren Flits.
	 */
	BufferQueue(unsigned int space=1);
    
	//! Der Destruktor.
	virtual ~BufferQueue() {};
    
	//! 
	/*!
	 * Liefert zurueck, ob noch Platz einer bestimmten Groesse in der Queue 
	 * zur Verfuegung steht.
	 *
	 * @param request Die Gr?sse des Bedarfes.
	 * @return true: Wenn Platz der entspr. Gr?sse vorhanden ist, false andernfalls.
	 */
	//virtual bool freeSpace(unsigned int request, unsigned int nextBuffer);
    	
	//! Vorderstes Flit der Queue.
	/*! 
	 * Liefert das Flit an der Spitze der Warteschlange zurueck.
	 *
	 * @return Vorderstes Flit oder NULL, falls Queue leer.
	 */
	virtual Flit* front() const;

	virtual Flit* back() const;
	
	//! Test auf leere Queue.
	/*!
	 * Diese Methode prueft, ob die Warteschlange aktuell leer ist.
	 * 
	 * @return true, falls Queue leer.
	 */
	virtual bool empty() const;

	//! Die aktuelle Aufnahmekapazitaet.
	/*!
	 * Diese Methode bestimmt die Zahl der Flits, die zur Zeit 
	 * garantiert aufgenommen werden koennten
	 *
	 * @return Aktuelle Aufnahmekapazitaet der Queue.
	 */
	virtual unsigned int freeSpace() const;
    	
	//! Die voraussichtliche Aufnahmekapazitaet.
	/*!
	 * In einem Takt kann maximal ein Flit ueber eine Leitung des Netzes gesendet
	 * und am Ziel eingepuffert werden. Blockierungen treten nur bei den Heads 
	 * auf. Da alle Komponenten eines Netzes verzoegerungsfrei arbeiten, kann bei
	 * vermitteltem Head eines Paketes davon ausgegangen werden, dass in jedem 
	 * Folgetakt ein weiteres Flit den Puffer verlaesst. Diese Methode bestimmt
	 * den vorraussichtlich verfuegbaren Platz unter dieser Annahme. 
	 * 
	 * @return Vorraussichtliche Aufnahmekapazitaet der Queue.
	 */
	virtual unsigned int estimatedSpace() const;
	
	//! Anzahl gepufferter Flits.
	/*! 
	 * Diese Methode bestimmt die Anzahl, der in dieser Warteschlange gepufferten
	 * Flits.
	 *
	 * @return Anzahl gepufferter Flits.
	 */
	virtual unsigned int spaceInUse() const;
	
	//! Maximale Anzahl von Flits.
	/*!
	 * Liefert die maximale Anzahl von Flits, die in dieser Warteschlange 
	 * gepuffert werden koennen.
	 * 
	 * @return Maximale Anzahl von Flits.
	 */
	virtual unsigned int queueSpace() const;
	 	
	//! Test auf aufgenommenes Flit.
	/*!
	 * Diese Methode prueft, ob im aktuellen Takt bereits ein Flit
	 * in die Queue aufgenommen wurde.
	 *
	 * @return true, falls in aktuellem Takt Flit aufgenommen wurde.
	 */
	inline bool Pushed() const
	{
		return pushed.valid();
	}

	//! Test auf entferntes Flit.
	/*!
	 * Diese Methode prueft, ob im aktuellen Takt bereits ein Flit
	 * aus der Queue entfernt wurde.
	 *
	 * @return true, falls in aktuellem Takt Flit entfernt wurde.
	 */
	inline bool Popped() const
	{
		return popped.valid();
	}

	// Wenn dieses Flag gesetzt ist, dann wird der Puffer so behandelt,
	// wie wenn er INTMAX/2 Flit-Pltze htte. 
	bool infinite;

	// Wenn diese Signatur gesetzt ist, dann wird das infinite-Flag am
	// Ende des Netzzyklusses zurckgesetzt.
	Signature cut;

protected:

	//! Fuellt die Queue mit einem Flit.
	/*!
	 * Mit dieser Methode wird ein Flit eines Paktes gepuffert. Vorher
	 * muss auf verfuegbaren Platz geprueft werden, ansonsten wird
	 * abgebrochen. Nach dem Puffern wird dem Senderegister signalisiert,
	 * dass ein neues Flit aufgenommen wurde.
	 * 
	 * @see RegisterContainer::announceFlit
	 * 
	 * @param flit Das zu puffernde Flit.
	 */
	virtual void push(Flit* flit);
	
	//! Vorderstes Flit aus der Queue entfernen.
	/*!
	 * Entfernt das Flit an der Spitze der Warteschlange. Anschliessend wird
	 * dem Senderegister mitgeteilt, dass vorderstes Flit entfernt wurde.
	 *
	 * @see RegisterContainer::eraseFlit
	 *
	 * @return Entferntes Flit oder NULL, falls Queue leer.
	 */
	virtual Flit* pop();
    
	//! Vorderstes Flit der Queue entfernen.
	/*!
	 * Wie BufferQueue::pop, nur ohne Einbeziehung des Senderegister.
	 * 
	 * @return Entferntes Flit oder NULL, falls Queue leer.
	 */
	virtual Flit* raw_pop();
	
	//! Pufferkapazitaet modifizieren.
	/*!
	 * Diese Methode setzt die maximale Anzahl von Flits, die verwaltet
	 * werden koennen auf einen Wert > 0. Die Anwendung ist nur zulaessig, 
	 * wenn die Queue leer ist.
	 *
	 * @return true, falls Anwendung erfolgreich.
	 */
	virtual bool resize(unsigned int size);

	//! Puffer komplett leeren.
	/*!
	 * Leert den gesamten Puffer und setzt das Senderegister zurueck.
	 */
	virtual void clear();

	//! Zugriff auf das Sende-Register.
	/*!
	 * Diese Methode liefert einen Pointer auf das Sende-Register
	 * dieser BufferQueue.
	 * 
	 * @return Das Ausgangsregister der Queue.
	 */
	inline RegisterContainer* Register() { return rc; }

private:

	//! Senderegister. 
	RegisterContainer* rc;	
	
	//! Maximale Anzahl von Flits in dieser Queue.
	unsigned int maxSpace;

	//! Interne Queue.
	std::list<Flit*> internalQueue;

	//! Markierung fuer entferntes Flit in aktuellem Takt.
	Signature popped;

	//! Markierung fuer aufgenommenes Flit in aktuellem Takt.
	Signature pushed;
};

#endif

