#ifndef __COMPONENT_H__
#define __COMPONENT_H__

#include <analysable.h>
#include <utility/xmlelement.h>
#include <utility/xmlattribute.h>
#include <utility/bitmask.h>
#include <utility/factory.h>

#include <sstream>
#include <string>
#include <stdio.h>


//! Die Komponenten eines Verbindungsnetzes.
/*!
 * Die Basisklasse fuer alle Komponenten eines Verbindungsnetzes definiert
 * Identifikationsmerkmale und die Maskierung erreichbarer Ziele. Eine
 * Komponente kann entweder eine Leitung oder ein Bauteil (Puffer, Switch) 
 * sein und besitzt ein ihr zugehoeriges Element im XML-Dokument einer 
 * Simulationsbeschreibung, ueber welches alle relevanten Parameter 
 * initialisiert werden koennen. Die in Senderichtung erreichbaren Ziele 
 * einer Komponente werden ueber eine Bitmaske festgelegt. Die Ziele eines
 * Verbindungsnetze werden eindeutig durch je ein Bit der Bitmaske definiert.
 * An einer Komponente koennen je nach Bedarf Messungen durchgefuehrt werden
 *
 * @see Bitmask
 * @see Analysable
 * @see XMLElement
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Component : public XMLElement, public Analysable, public FactoryElement
{
public:
	
	//! Die Identifikationen der Komponenten.
	enum e_rtti_value{ SOURCE, BUFFER, TARGET, ROUTER, WIRE };
		
	//! Konstruktor.
	Component() : XMLElement(NULL), _index(0) {}

	//! Laufzeit-Typinformation der Komponente.
	/*!
	 * Die einzelnen Komponenten eines Verbindungsnetzes haben unterschiedliche 
	 * Eigenschaften. Diese Methode erlaubt es, eine Komponente zu identifzieren
	 * und entsprechend ihrer Eigenschaften zu behandeln.
	 * 
	 * @see e_rtti_value
	 * 
	 * @return Identifikation der Komponente.
	 */
	virtual const int rtti() const = 0;

	//! Name der Komponente.
	/*!
	 * Fuer die Ausgabe von Informationen zu einer Simulation ist die Bezeichnung
	 * von Komponenten angebracht anstatt die Identifikationsnummer zu verwenden.
	 * 
	 * @return Name der Komponente.
	 */
	virtual const std::string Name() const = 0;
		
	//! Rekursive Generierung von Bitmasken. 
	/*!
	 * Zu Beginn einer Simulation muessen die Bitmasken der Komponenten anhand 
	 * der Struktur des Verbindungsnetzes initialisiert werden. In der Regel
	 * ergibt sich die Bitmaske einer Komponente aus der Bitmaske nachfolgender
	 * Komponenten in Senderichtung. Diese Methode soll dazu verwendet werden,
	 * die Generierung der Bitmasken nachfolgender Komponenten anzustossen und
	 * daraus die Bitmaske dieser Komponente zu bestimmen.
	 *
	 * @return Generierte Bitmaske dieser Komponente.
	 */
	virtual const Bitmask& generateBitmask() = 0;

	//! Zuruecksetzen einer Komponente.
	/*!
	 * Bei transienten Simulationen oder Rekonfigurationen, kann es notwendig
	 * sein, die Komponenten eines Verbindungsnetzes zurueck in ihren initialen
	 * Zustand versetzen zu koennen.
	 */
	virtual void reset() = 0;
	
	//! Interner Name der Komponente.
	/*!
	 * Jede Komponente besitz in ihrer Eigenschaft als XMLElement ein Attribut
	 * index, dessen Wert eine wichtige Informationen zu einer Komponente
	 * darstellt. Zusammen mit dem Namen laesst sich eine Komponente damit
	 * eindeutig in der XML-Beschreibung einer Simulation identifizieren.
	 * 
	 * @see XMLElement
	 * 
	 * @return Wert des XML-Attributes index.
	 */
	inline const std::string InternalName() const
	{
		return getAttribute("index");
	}
	
	//! Index der Komponente.
	/*!
	 * Komponenten werden in einer Konfiguration verwaltet und dort in
	 * Listen mit Komponenten gleichen Typs (RTTI) verwaltet. Der Index
	 * einer Komponente beschreibt die Position in der jeweiligen Liste.
	 *
	 * @see Configuration
	 * 
	 * @return Index der Komponente.
	 */
	inline unsigned int Index() const
	{
		return _index;
	}

	//! String-Repraesentation.
	/*!
	 * Alle Informationen zu einer Komponente in einem String.
	 * 
	 * @return String-Repraesentation der Komponent.
	 */
	inline const std::string toString() const
	{
		char tmp[sizeof(unsigned int)*8+1];
		sprintf(tmp, "%u", Index());
		return Name() + " " + tmp + " [" + InternalName() + "]";
	}

	//! Lesen der Bitmaske einer Komponente.
	/*!
	 * Fuer die Vermittlung von Nachrichten in einem Verbindungsnetz werden
	 * die Bitmasken der Komponenten mit den Bitmasken der Nachrichten
	 * verglichen. Mit dieser Methode kann auf die Bitmaske dieser Komponente
	 * zugegriffen werden.
	 * 
	 * @return Bitmaske der Komponente.
	 */
	inline const Bitmask& getBitmask() const 
	{
		return _bitmask; 
	}

protected:

	//! Setzt den Index der Komponente.
	/*!
	 * Diese Methode setzt den Index einer Komponente. Das geschieht 
	 * normalerweise nur beim Parsen einer Netzkonfiguration.
	 *
	 * @param Index Der neue Wert fuer den Index der Komponente.
	 */
	void setIndex(unsigned int Index)
	{
		_index = Index;
	}
	
	//! Die Bitmaske dieser Komponente.
	/*!
	 * Initial ist die Bitmaske leer, d.h. es sind kein Bits gesetzt. Der
	 * (einmalige) Aufruf von generateBitmask setzt die Bits entsprechend
	 * der erreichbaren Ziele.
	 * 
	 * @see Bitmask
	 */
	Bitmask _bitmask;

private:

	//! Der Index der Komponente.
	unsigned int _index;
};

#endif
