#ifndef __DISTRIBUTION_H__
#define __DISTRIBUTION_H__

#include <stdexcept>
#include <sstream>
#include <vector>
#include <map>
#include <macros.h>
#include <utility/factory.h>
#include <utility/xmlelement.h>
#include <mathparser/ParserImpl.h>
#include <algorithm>


//! Struktur f�r Mapping von Index-String auf Vektor-Position
typedef std::map<std::string, unsigned int> IndexMap;

class CumulativeException : public std::runtime_error
{
public:
        
	CumulativeException(const std::string &msg) : runtime_error(msg) {}
};

template<typename _Tp>
class CumulativeArray
{
public:

	//! Standardkonstruktor.
	CumulativeArray() : array(NULL), array_size(0), sum(0), norm(0) {}

	//! Datenkonstruktor.
	template<typename _C>
	// VORSICHT! keine adresse uebergeben
	CumulativeArray(_C container)
	{
		norm = 1; // standard norm
		sum = 0;
		array = new _Tp[array_size=container.size()];
		std::transform(container.begin(),container.end(),array,cumulative(sum));
	}
	
	/*! Destructor. */
	~CumulativeArray()
	{
		delete[] array;
	}

	//! Gets the sum of elements
	const _Tp& Sum() const
	{
		return sum;
	};
	//! Returns the size of the underling array
	const unsigned int& size() const
	{
		return array_size;
	};
	//! Sets the norm value
	const void setNorm(_Tp norm)
	{
		this->norm = norm;
	};

	// returns the norm
	const _Tp getNorm() const 
	{
		return norm;
	};
	
	//! Get an element
	const _Tp& operator[] (unsigned int elem) const
	{
		return array[elem];
	};
	
	//! Assignment
	CumulativeArray& operator= (const CumulativeArray& carray)
	{
		sum = carray.sum;
		norm = carray.norm;
		
		if(array_size != carray.array_size)
		{
			if(array)
				delete[] array;

			array_size = carray.array_size;
			array = new _Tp[array_size];
		}

		for(unsigned int i=0; i<array_size; i++)
			array[i] = carray.array[i];

		return (*this);
	};

	//! check if normalized
	const bool normalized() const
	{
		return sum == norm;
	};

	// remove a value and fit the cum values
	const void remove(unsigned int elem) 
	{
		// new sum
		double diff = elem ? array[elem]-array[elem-1] :  array[elem]; 
		sum -= diff;
		// new norm
		norm = sum;
		--array_size;
		
		while(elem < array_size) {
			array[elem] = array[elem+1]-diff;
			elem++;
		}
	}
	
	//! normalisieren
	const void normalize()
	{
		std::for_each(&array[0], &array[array_size], divByNorm(norm, sum));
		sum = norm;
	};

	const unsigned int choose_higher(const _Tp border)
	{
		if (!normalized())
			throw CumulativeException("cumulative values are not normailized");
			
		if (border > norm) 
		{
			std::stringstream s;
			s << "you can either select a random number between 0 and " 
				<< norm << " or change the norm and normalize";
			throw CumulativeException(s.str());
		}
		for (unsigned int i=0; i<array_size; i++)
			if (array[i] > border) return i;
		
		_ERROR("Cannot choose an element out of distributed elements, because there are only elements left, having zero probability");
	};

private:

	_Tp*  array; // helper array, copied container
	unsigned int array_size; // the arrays size

	_Tp sum; // summed up elemets
	_Tp norm; // the norm

protected:

	//TODO: diese beiden zusammenfuegen und operatoren uebergeben
	class divByNorm : public std::unary_function<_Tp, _Tp>
	{
	public:
	
		divByNorm(_Tp& _norm, _Tp& _sum) : _norm(_norm), _sum(_sum) {}

		_Tp operator()(double &val)
		{
			val=val*_norm/_sum;
			return val;
		}
		
	private:
	
		_Tp& _norm;
		_Tp& _sum;
	};

	class cumulative : public std::unary_function<_Tp, _Tp>
	{
	public:
		cumulative(_Tp& _sum) : _sum(_sum) {}

		_Tp operator()(double &val)
		{
			_sum += val;
			val = _sum;
			return val;
		};

	private:
		_Tp& _sum;
	};
};

template<typename T>
std::ostream& operator<<(std::ostream & out, CumulativeArray<T>& carray)
{
	for (unsigned int i = 0; i<carray.size(); i++) 
	{
		out << carray[i];
		if (i+1<carray.size())
			out << " ";
	}
	return out;
}


//! Basisklasse fuer Verteilungsfunktionen.
/*!
 * Diese Klasse ist Ausgangspunkt fuer zahlreiche Verteilungsklassen, die
 * zur Spezifikation von Multicast, raeumlichen Verteilungen und Verteilungen
 * von Paketprioritaeten verwendet werden.
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Distribution : public FactoryElement, public XMLElement, public ParserImpl 
{
public:

	//! Standard-Konstruktor.
	/**
	 * Die internen Strukturen werden auf initiale Werte gesetzt.
	 */
	Distribution();

	//! Destruktor.
	virtual ~Distribution();

	//! Waehlt eine Zahl zwischen 0 und DistElements()-1 mit der vorgegebenen Verteilung
	virtual int choose();

	//! Zuletzt gezogenes Element entfernen.
	virtual void remove_last ();

	//! Initialisierung der kumulierten Wahrscheinlichkeiten.
	void initArray(std::vector<double> &values);
	
	//! Test auf erfolgte Initialisierung.
	/*!
	 * Die Implementierung der Basisklasse prueft, ob ein Aufruf von
	 * initArray erfolgt ist.
	 *
	 * @return true, falls kumulierte Wahrscheinlichkeiten initialisiert.
	 */
	virtual bool Initialized() const
	{
		return initialized;
	}
	
	//! Zuruecksetzen der internen Strukturen.
	/*!
	 * Die Implementierung der Basisklasse setzt das Array der kumulierten
	 * Wahrscheinlichkeiten in seinen initialien Zustand zurueck.
	 */
	virtual void reset();

	//! Relative Wahrscheinlichkeit fuer Element der Verteilung.
	virtual double probability(unsigned int distelem) const;

	//! Praefix fuer Attribute.
	void setAttributePrefix(const char* prefix);

	//! Prefix fuer Attribute
	const char* attributePrefix() const { return attrprefix; }

	//! Die Anzahl der insgesamt zu verteienden Elemente.
	unsigned int distElements() const
	{
		return distelements;
	}

	//! Die Anzahl der verbliebenen Elemente.
	unsigned int distElementsLeft() const
	{
		return id.size();
	}

	//! Legt die Anzahl der zu verteilenden Elemente fest
	void setDistElements(unsigned int nr)
	{
		distelements = nr;
		for (unsigned int i=0; i<distelements; i++)
			id.push_back(i);
	}

	//! Zugriff auf Mapping Index-String zu interner Position
	IndexMap& indexMapping()
	{
		return indexmap;
	}
	
protected:

	//! Anzahl der zu verteilenden Elemente.
	unsigned int distelements;

	//! Aufsteigende Liste von zu verteilenden Elementen.
	std::vector<unsigned int> id;
	
	//! Zuletzt gezogenes Element.
	int last;
		
	//! Zuordnung bestimmter Indizes zu verteilbaren Elementen.
	IndexMap indexmap;

private:	

	//! Flag fuer Initialsierung der kumulierten Wahrscheinlichkeiten.
	bool initialized;
	
	//! Prefix fuer Attribute.
	char* attrprefix;

	//! Kumulierte Wahrscheinlichkeiten. 
	CumulativeArray<double> *carray;

	//! Backup fuer kumulierte Wahrscheinlichkeiten.
	CumulativeArray<double> backup;
	
	//! Ausgangswerte fuer kumulierte Wahrscheinlichkeiten.
	std::vector<double> varray;

	//! Summe der Ausgangswerte fuer kumulierte Wahrscheinlichkeiten.
	double varray_sum;
};

// Ausgabe der Distribution auf einen ostream
inline std::ostream& operator<<(std::ostream & out, Distribution* d)
{
	for (unsigned int i=0; i<d->distElements(); i++)
		out << d->probability(i) << " ";
	return out;
}

#endif // DISTRIBUTION_H
