#ifndef __ENVIRONMENT_H__
#define __ENVIRONMENT_H__

#include <utility/factory.h>
#include <utility/configdirectives.h>

#include <cassert>

class CmdParser;


//! Laufzeitumgebung.
/*!
 * Diese Klasse dient zur Initialisierung der Laufzeitumgebung eines
 * CINSim-Programmes. Es werden Kommandozielen-Optionen und die 
 * Konfigurationsdatei geparst und eine Dictonary abgelegt. 
 * Anschliessend koennen spezifische Aktionen fuer jedes Programm 
 * durchgefuehrt werden. Am Ende der Laufzeit kann die Umgebung dann 
 * wieder zurueckgesetzt werden.
 *
 * @see ConfigDirectives
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Environment : public ConfigDirectives, public FactoryElement
{
	
friend class Sequencer;
friend int main(int,char**);


public:

	//! Zugriff auf Laufzeitumgebung.
	/*!
	 * Es existiert fuer jedes Programm genau eine Laufzeitumgebung, die
	 * ueber Setup initialisiert und Halt zurueckgesetzt werden kann.
	 * 
	 * @see Setup
	 * @see Halt
	 *
	 * @return Instanz der Klasse Environment oder NULL falls keine existiert.
	 */
	static const Environment* Instance()
	{
		return _instance;
	}

	
protected:

	//! Geschuetzter Konstruktor.
	Environment() {}

	//! Laufzeitumgebung initialisieren.
	/*!
	 * Es wird eine Instanz der Laufzeitumgebung ueber die Klassen-Factory
	 * erzeugt, die Kommandozeilen-Optionen und die Konfigurationsdatei geparst 
	 * und die Informationen im Dictonary der Umgebung abgelegt. Anschliessend 
	 * werden die fuer die erzeugte Instanz notwendigen Initialisierungschritte 
	 * durchgefuehrt.
	 *
	 * @see initialize
	 *
	 * @param id Identifier fuer Klassen-Factory.
	 * @param argc Anzahl der Kommandozeilenargumente.
	 * @param argv Kommandozeilenargumente.
	 */
	static void Setup(const char* id, int argc, char** argv);

	//! Laufzeitumgebung zuruecksetzen.
	/*!
	 * Die fuer die verwendete Instanz notwendigen Schritte am Ende der
	 * Laufzeit werden duchgefuehrt und die Instanz freigegeben.
	 * 
	 * @see finalize
	 */
	static void Halt();
	
	//! Parameter fuer Simulationssequenz aktualisieren.
	/*!
	 * Fuer Simulationssequenzen koennen ueber diese Methode Parameter in das
	 * Dictonary eingetragen oder aktualsiert werden.
	 *
	 * @see Sequencer
	 *
	 * @param name Name des Parameters.
	 * @param value Neuer Wert fuer Parameter.
	 */
	static void setParameter(std::string name, double value);
	
	//! Initialisierung der Umgebung.
	/*!
	 * Fuer jede Laufzeitumgung muessen ueber diese Methode die notwendigen
	 * Inititialisierungen vorgenommen werden. Der Aufruf erfolgt nach Parsen
	 * der Kommandozeilenoptionen und Konfigurationsdatei.
	 * 
	 * @param argc Anzahl der Kommandozeilenargumente.
	 * @param argv Kommandozeilenargumente.
	 * @return true, falls Initilisierung erfolgreich.
	 */
	virtual bool initialize(int argc, char** argv);

	//! Zuruecksetzen der Umgebung.
	/*!
	 * Am Ende der Laufzeit wird durch Aufruf dieser Methode die Umgebung
	 * zurueckgesetzt.
	 * 
	 * @return true, falls Zuruecksetzen erfolgreich.
	 */
	virtual bool finalize() = 0;

	//! Parser fuer Kommandozeilenoptionen.
	/*!
	 * Jede Laufzeitumgebung stellt ueber diese Methode einen eigenen Parser fuer 
	 * Kommandozeilenoptionen bereit.
	 *
	 * @return Zu verwendender Kommandozeilenparser.
	 */
	virtual CmdParser* createCmdParser() const = 0;

	
private:

	//! Die Instanz der Laufzeitumgebung.
	static Environment* _instance;
};


inline const Environment* Env()
{
	assert(Environment::Instance());
	
	return Environment::Instance();
}

#endif
