#ifndef __ANALYSIS_H__
#define __ANALYSIS_H__

#include <utility/factory.h>
#include <string>
#include <cfloat>

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

using std::string;

//! Stochastische Auswertung von Simulationen mit CINSim.
/*!
 * Abstrakte Basisklasse f�r verschiedene Implementierungen der stochastischer
 * Auswertung von Simulationslaeufen. Es existieren eine Reihe statischer
 * Methoden fuer die Verwendung innerhalb einer Simualtion. Die eigentliche 
 * Instanz der Auswertung ist nicht direkt modifizierbar, wodurch verteilte
 * Simulationen ueber MPI unabhaenig von der Implementierung der Auswertung
 * realisierbar sind.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Analysis : public FactoryElement
{

friend class XmlSimulation;	// declare
friend class CINSimEnv; // create
friend class Sequencer; // reset
friend int main(int,char**);

public:

	//! Ergebnis der Auswertung fuer eine Messung.
	/*!
	 * Fuer jede durchzufuehrende Messung wird eine solche Datenstruktur fuer
	 * deren Ergebnisse erstellt.
	 *
	 * @see Declare
	 */
	struct Result
	{
		//! Konstruktor.
		/*!
		 * Erzeugt Ergebnisstruktur mit initialen Werten. Der Fehler ist maximal, 
		 * so dass Ergebniss noch nich akzeptabel ist.
		 *
		 * @see acceptable
		 */
		Result() : precision(0), confidence(0) { reset(); } 
		
		//! Ueberpruefung des aktuellen Ergebnisses.
		/*!
		 * Ein Ergebnis gilt als akzeptabel, falls der Fehler die geforderte
		 * Genauigkeit uebersteigt.
		 *
		 * @return true, falls Ergebnis akzeptabel.
		 */
		inline bool acceptable() const
		{
			return error < precision;
		}

		//! Aktuelle Ergebnisse verwerfen.
		/*!
		 * Alle Ergebnisse werden verworfen und Fehler auf maximalen Wert 
		 * gesetzt. Geforderte Konfidenz und Genauigkeit werden beibehalten.
		 */
		inline void reset()
		{
			chkpnts = 0.0;
			values = 0.0;  
			discarded = 0.0;
			estimate = 0.0;
			variance = 0.0;
			freedom = 0.0; 
			
			error = DBL_MAX;
		}
		
		//! Anzahl der Checkpoints.
		double chkpnts;
		//! Anzahl der Messwerte.
		double values;
		//! Verworfene Messwerte.
		double discarded;
		//! Abschaetzung.
		double estimate;
		//! Varianz fuer Abschaetzung.
		double variance;
		//! Freiheitsgrade.
		double freedom;
		
		//! Fehler der Abschaetzung.
		double error;
		//! Geforderte Genauigkeit.
		double precision;
		//! Geforderte Konfidenz.
		double confidence;
	};

	
	//! Information zu Messung.
	/*!
	 * Zu Beginn einer Analyse werden alle Messungen ueber eine solche
	 * Datenstruktur spezifiziert.
	 * 
	 * @see Declare
	 */
	struct VariableInfo
	{
		//! Konstruktor fuer Defaults.
		VariableInfo() : type(Mean), quantile(0.5), precision(0.05), confidence(0.95), 
						 independent(false), initialtransient(true) {}
		
		//! Art der Auswertung (Mittelwert oder Quantil)
		enum { Mean, Quantile } type;
		//! Parameter fuer Quantil.
		double quantile;
		//! Abbruchkriterium.
		double precision;
		//! Konfidenzniveau.
		double confidence;
		//! Flag fuer unabhaengige (IID) Messwerte.
		bool independent;
		//! Flag fuer Existenz einer initialen transienten Phase.
		bool initialtransient;
	};

	
	//! Globale Instanz der Klasse Analysis.
	/*!
	 * Innerhalb einer Simulation wird ganau eine Instanz der Klasse
	 * Analysis benoetigt, um die Messdaten einer Simulation auszuwerten.
	 * Diese Methode liefert dieses Instanz, welche aber zuvor einmalig
	 * �ber die Klassen-Factory erstellt werden muss.
	 *
	 * @see FactoryElement
	 * @see createInstance
	 * 
	 * @return Globale Instanz der stochastischen Auswertung oder NULL.
	 */
	inline static const Analysis* Instance()
	{
		return _instance;
	}
	
	//! Spezifikation des Identifiers fuer Klassen-Factory.
	/*!
	 * Instanzen der Klasse Analysis werden ueber eine Klassen-Factory erstellt,
	 * wobei eine Identifzierung der zu verwendenden Implementierung ueber
	 * einen String erfolgt. Diese Methode setzt diesen String. Der Aufruf
	 * muss erfolgen, bevor eine Instanz generiert wurde.
	 *
	 * @pre Instance() == NULL
	 * 
	 * @param id Identifier fuer Klassen-Factory.
	 */	
	static void setFactoryId(const string& id);

	//! Identifier fuer Klassen-Factory.
	/*!
	 * Diese Methode liefert den Identifier fuer die Klassen-Factory zur
	 * Erstellung einer Instanz der Klasse Analysis.
	 *
	 * @see createInstance
	 *
	 * @return Identifier fuer Klassen-Factory.
	 */
	static string FactoryId()
	{
		return _factoryid;
	}
	
	//! Neuer Messwert fuer stochastischen Auswertung.
	/*!
	 * Die erzeugten Messungen der Simulationen werden einzelnen Variablen der 
	 * stochastischen Auswertung zugeordnet. Diese Methode uebergibt einen
	 * neuen Messwert f�r eine der Variablen.
	 * 
	 * @pre Instance() != NULL
	 * 
	 * @see Declare
	 * 
	 * @param variable
	 * @param value
	 */
	static void Advance(unsigned int variable, double value);

	//! Neue Werte aufnehmen.
	/*!
	 * Bevor neue Werte zur Analyse aufgenommen werden, muss dies ueber diese
	 * Methode eingeleitet werden. Bei verteilten Simulationen wird zunaechst
	 * geprueft, ob einzelne Messungen bereits ihr (globales) Konfidenzintervall
	 * erreicht haben.
	 *
	 * @see StopGathering
	 */
	static void StartGathering();

	//! Neue Werte aufgenommen.
	/*!
	 * Nachdem alle neuen Werte der Analyse uebergeben wurden, schliesst der
	 * Aufruf dieser Methode den Sammelprozess ab. Bei verteilten Simulationen
	 * wird nun die globale Abschaetzung der einzelnen Messungen aktualisiert.
	 * Hat eine Messung ihr Konfidenzintervall erreicht, so werden alle 
	 * verteilten Simulationen benachrichtigt.
	 *
	 * @see StartGathering
	 */
	static void StopGathering();
	
	//! Anzahl durchzufuehrender Messungen.
	/*!
	 * Fuer die Auswertung von Simulationsergebnissen wird eine feste Anzahl
	 * von Messvariablen definiert, die Werte von Simulation erhalten. Diese 
	 * Methode liefert die festgelegte Anzahl. 
	 * 
	 * @see Declare
	 * @return Anzahl definierter Messvariablen.
	 */
	inline static unsigned int Declared()
	{ 
		return _declared; 
	}

	//! Auswertung abgeschlossen.
	/*!
	 * Die Auswertung gilt als abgeschlossen, wenn fuer alle Messungen das
	 * Konfidenzintervall erreicht wurde.
	 *
	 * @return true, falls Konfidenzintervall fuer alle Messungen erreicht.
	 */
	inline static bool Finished()
	{
		return _finished == _declared;
	}
	
	//! Aktuelles Ergebnis fuer Messung.
	/*!
	 * Diese Methdode liefert das aktuelle Ergebnis zu einer Messung.
	 *
	 * @pre index 0 < index =< Declared()
	 * @see Declared
	 * @param index Der Index der Messung (Indizes beginnen bei 1).
	 * @return Aktuelles Ergebnis fuer Messung zu spezifiziertem Index.
	 */
	static const Result& CurrentResult(unsigned int index);

	//! Auswertung initialisieren.
	/*!
	 * Bevor Messwerte aufgenommen und analysiert werden koennen, muss ueber 
	 * diese Methode die stochastische Auswertung einmalig initialisiert werden.
	 * Ein erneuter Aufruf mit soll dafuer sorgen, dass die Auswertung reinitialsiert
	 * wird.
	 * 
	 * @param list Liste mit Informationen zu durchzufuehrenden Messungen.
	 * @param count Anzahl der durchzufuehrenden Messungen.
	 * @return false, falls erfolgreich.
	 */
	virtual bool declareVariables(const VariableInfo* list, unsigned int count) = 0;

	//! Messung an Auswertung uebergeben.
	/*!
	 * Ueber diese Methode wird eine neuer Messwert an die Auswertung uebergeben
	 * und das zugehoerige Ergebnis gegebenenfalls aktualisiert. Die Identifier
	 * der Messungen beginnen bei 1.
	 *
	 * @param variable Identifier fuer Messung.
	 * @param value Neuer Messwert.
	 * @param[out] result Ergebnisdatenstruktur fuer Messung.
	 * @return true, falls Ergebnis aktualisiert wurde.
	 */
	virtual bool advanceVariable(unsigned int variable, double value, Result& result) = 0;

	//! Messung fuer eine Variable beenden.
	/*!
	 * Wenn das Ergebnis einer Messung das festgelegte Konfidenzintervall mit 
	 * geforderter Genauigkeit erreicht hat, so wird dies der Auswertung ueber 
	 * diese Methode mitgeteilt.
	 *
	 * @para variable Index der abgeschlossenen Messung.
	 */
	virtual void finishVariable(unsigned int variable) = 0;
	
		
protected:
	
	//! Geschuetzter Konstruktor.
	Analysis() {}

	//! Auswertung initialisieren.
	/*!
	 * Ueber diese Methode werden die durchzufuehrenden Messungen spezifiziert
	 * und damit die stochastische Auswertung einmalig initialisiert.
	 *
	 * @pre Instance != NULL
	 *
	 * @see VariableInfo
	 * 
	 * @param list Liste mit Informationen zu Messungen.
	 * @param count Laenge der Liste.
	 */
	static void Declare(const VariableInfo* list, unsigned int count);
	
	//! Alle Ergebnisse zuruecksetzen.
	/*!
	 *
	 * @pre Instance() != NULL
	 * @ore Declared() != 0
	 * 
	 * Diese Methode versetzt die Auswertung in ihren initialen Zustand, d.h
	 * die Ergebnisse aller Messungen verden verworfen.
	 */
	static void Reset();
	
	//! Erzeugen einer Instanz ueber Klassen-Factory.
	/*!
	 * Eine Instanz der Klasse Analysis wird ueber diese Methode generiert.
	 * Zuvor kann der Identifier fuer die Klassen-Factory gesetzt werden.
	 * 
	 * @see setFactoryId
	 * 
	 * @return true, falls Instanz erstellt wurde.
	 */
	static bool createInstance();
	

private:
	
	//! Anzahl durchzufuehrender Messungen.
	static unsigned int _declared;
	//! Anzahl abgeschlossener Messungen.
	static unsigned int _finished;
	//! Liste mit Variableninformationen.
	static VariableInfo* _list;
	//! Liste mit Ergebnissen zu Messungen.
	static Result* _results;
	//! Verwendete stochastische Auswertung.
	static Analysis* _instance;
	//! Factory-Id fuer stochastische Auswertung.
	static string _factoryid;
	//! Status-Flag fuer Aufnahme neuer Werte.
	static bool _gathering;
};

#endif
