#ifndef __PRNG_H__
#define __PRNG_H__

#include <utility/factory.h>
#include <cmath>
#include <limits.h>
#include <stdlib.h>

using std::string;

//! Generator fuer Pseudo-Zufallszahlen.
/*!
 * Diese Klasse definiert ein gemeinsames Interface fuer alle Quellen von 
 * Pseudo-Zufallszahlen, die in CINSim zur Verfuegung stehen. Innerhalb einer
 * Simulation bzw. eines Simulationsprozesses existiert genau  eine 
 * Instanz dieser Klasse. Fuer verschiedene Prozesse innerhalb verteilter
 * Simulationen koennen unabhaengige Streams von Zufallszahlen generiert
 * werden.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class PRNG : public FactoryElement
{

friend class Test_PRNG;
friend class CINSimEnv;
friend int main(int,char**);

public:
	
	//! Generator fuer Zufallszahlen.
	/*!
	 * Innerhalb einer Simulation bzw. eines Simulationsprozesses existiert
	 * genau eine Instanz der Klasse PRNG als Quelle fuer Zufallszahlen.
	 * Diese Methode realisiert den Zugriff auf diese Instanz, die allerdings
	 * zuvor einmalig erzeugt werden muss.
	 *
	 * @see createInstance
	 * @see setFactoryId
	 * @see setStreamId
	 *		 
	 * @return Instanz der Klasse PRNG als Generator fuer Zufallszahlen, oder 
	 * NULL falls noch keine Instanz existiert.
	 */
	inline static PRNG* Instance()
	{	
		return _instance;
	}
		
	//! Spezifikation des Identifiers fuer Klassen-Factory.
	/*!
	 * Instanzen der Klasse PRNG werden ueber eine Klassen-Factory erstellt,
	 * wobei eine Identifzierung der zu verwendenden Implementierung ueber
	 * einen String erfolgt. Diese Methode setzt diesen String. Der Aufruf
	 * muss erfolgen, bevor eine Instanz generiert wurde.
	 * 
	 * @param id Identifier fuer Klassen-Factory.
	 */
	static void setFactoryId(const string& id);
	
	//! Identifier fuer Klassen-Factory.
	/*!
	 * Diese Methode liefert den spezifizierten Identifier fuer die
	 * Factory.
	 *
	 * @see setFactoryId()
	 * 
	 * @return Identifier fuer Klassen-Factory.
	 */
	inline static string FactoryId() 
	{ 
		return _factoryid; 
	}
	
	//! Identifier fuer Stream von Zufallszahlen.
	/*!
	 * @see setStreamId() 
	 * @return Identifier fuer verwendeten Stream von Zufallszahlen.
	 */
	inline static unsigned int StreamId() 
	{ 
		return _streamid; 
	}
	
	//! Zufaelliger Integer aus Intervall [start, end].
	/*!
	 * Diese Methode wird von allen abgeleiteten Klassen implementiert
	 * und liefert zufaelligen (vorzeichenlose) Integer zwischen start und
	 * end (beide eingeschlossen). start <= end wird vorausgesetzt.
	 *
	 * @return Zufaelliger Integer aus Intervall [start,end].
	 */
	virtual unsigned int uniformInt(unsigned int start, unsigned int end)=0;

	//! Zufaellige Gleitkommazahl aus Intervall ]0,1[.
	/*!
	 * Diese Methode liefert eine zufaellige Gleitkommazahl doppelter 
	 * Genauigkeit zwischen 0 und 1 (beide ausgeschlossen). PRNG_MAX_DBL_01
	 * und PRNG_MIN_DBL_01 spezifieren den groessten bzw. kleinsten zufaelligen 
	 * Wert.
	 *
	 * @return Zufaellige Gleitkommazahl aus Intervall ]0,1[.
	 */
	inline double uniformDouble()
	{
		return ((double)uniformInt(1,UINT_MAX-1))/(double)UINT_MAX;
	}

	//! Zufaellige Gleitkommazahl aus Intervall [0,1].
	/*!
	 * Diese Methode liefert eine zufaellige Gleitkommazahl doppelter 
	 * Genauigkeit zwischen 0 und 1 (beide eingeschlossen).
	 *
	 * @return Zufaellige Gleitkommazahl aus Intervall [0,1].
	 */
	inline double uniformDouble_01()
	{
		return ((double)uniformInt(0,UINT_MAX))/(double)UINT_MAX;
	}

	//! Zufaellige Gleitkommazahl aus Intervall [0,1[.
	/*!
	 * Diese Methode liefert eine zufaellige Gleitkommazahl doppelter 
	 * Genauigkeit zwischen 0 (eingeschlossen) und 1 (ausgeschlossen). 
	 * PRNG_MAX_DBL_01 spezifiert dabei den groessten zufaelligen Wert.
	 *
	 * @return Zufaellige Gleitkommazahl aus Intervall [0,1[.
	 */
	inline double uniformDouble_0()
	{
		return ((double)uniformInt(0,UINT_MAX-1))/(double)UINT_MAX;
	}

	//! Zufaellige Gleitkommazahl aus Intervall ]0,1].
	/*!
	 * Diese Methode liefert eine zufaellige Gleitkommazahl doppelter 
	 * Genauigkeit zwischen 0 (ausgeschlossen) und 1 (eingeschlossen). 
	 * PRNG_MIN_DBL_01 spezifiert dabei den kleinsten zufaelligen Wert.
	 *
	 * @return Zufaellige Gleitkommazahl aus Intervall ]0,1].
	 */
	inline double uniformDouble_1()
	{
		return ((double)uniformInt(1,UINT_MAX))/(double)UINT_MAX;
	}
	
	//! Zufaellige Gleitkommazahl aus Intervall [start, end].
	/*!		 
	 * Diese Methode liefert eine zufaellige Gleitkommazahl doppelter 
	 * Genauigkeit zwischen start und end (beide eingeschlossen).
	 *
	 * @return Zufaellige Gleitkommazahl aus Intervall [start,end].
	 */
	inline double uniformDouble(double start, double end)
	{
		return start + (end-start) * uniformDouble_01();
	}

	
	//! Exponentialverteilung E(lamda).
	/*!
	 * Diese Methode liefert mit Parameter \a lambda exponoential-verteilte 
	 * Zufallszahlen.
	 * 
	 * @note Mittelwerrt = 1/lambda
	 * @note p-Quantil = -(1/lambda)*ln(1-p)
	 *
	 * @param lambda Formparameter.
	 * @return Zufaellige Gleitkommazahl aus Intervall ]0,MAX_DBL]
	 */
	inline double exponential(double lambda)
	{
		return -(1.0/lambda) * log(uniformDouble_01());
	}

	//! Normalverteilung N(mean,variance).
	/*!
	 * Diese Methode liefert normal-verteilte Zufallszahlen mit Mittelwert
	 * \a mean und Varianz \a variance.
	 *
	 * @note p-Quantil = mean + z_p*sigma (z_p = p-Quantil der Standard-NV)
	 *
	 * @param mean Mittelwert der Verteilung.
	 * @param variance Varianz der Verteilung.
	 * @return Zufaellige Gleitkommazahl
	 */
	double normal(double mean, double variance);
	
	
protected:

	//! Konstruktor.
	PRNG() { }
	
	//! Eindeutige ID fuer Stream von Zufallszahlen.
	/*!
	 * In parallelen Simulationen kann mit dieser Methode den Generatoren
	 * der verschiedenen Simulationsprozesse eine eindeutige Identifikation
	 * zugewiesen werden. Anhand dieser Identifikation wird ein Stream von
	 * Zufallszahlen generiert, der unabh�ngig von Str�men anderer Generatoren
	 * mit abweichenden Identifikationen ist. Diese Methode wird von allen
	 * abgeleiteten Klassen implementiert und setzt den implementierten
	 * Generator an den Beginn des spezifizierten Streams. Der Aufruf sollte
	 * nach Moeglichkeit nicht direkt sonderm ueber setStreamId erfolgen.
	 *
	 * @see setStreamId
	 * 
	 * @param id Identifier fuer Stream von Zufallszahlen.
	 */
	virtual void uniqueStreamId(unsigned int id)=0;

	
	//! Erzeugen einer Instanz ueber Klassen-Factory.
	/*!
	 * Eine Instanz der Klasse PRNG wird ueber diese Methode generiert.
	 * Zuvor koennen die Identifier fuer die Klassen-Factory und der
	 * zu benutzenden Stream gesetzt werden.
	 * 
	 * @see setFactoryId
	 * @see setStreamId
	 * 
	 * @return true, falls Instanz erstellt wurde.
	 */
	static bool createInstance();
	
	
	//! Spezifikation des Identifiers fuer Stream von Zufallszahlen.
	/*!
	 * Eine Instanz der Klasse PRNG sollte unabhaengige Streams von 
	 * Zufallszahlen bereitstellen, die beispielsweise fuer verteilte 
	 * Simulationen verwendet werden koennen. Ein einzelner Stream  
	 * wird ueber einen ganzzahligen Identifier spezifiziert und ueber
	 * diese Methode gesetzt.
	 * 
	 * @see uniqueStreamId
	 * 
	 * @pram id Identifier fuer Stream von Zufallszahlen.
	 */
	static void setStreamId(unsigned int id);

	
private:

	//! Identifier fuer generierten Strom von Zufallszahlen.
	static unsigned int _streamid;

	//! Identifier fuer Klassen-Factory.
	static string _factoryid;

	//! Globale Instanz. 
	static PRNG* _instance;
};

#define PRNG_MAX_DBL_01 (((double)UINT_MAX-1.0)/(double)UINT_MAX)
#define PRNG_MIN_DBL_01 (1.0/(double)UINT_MAX)

inline double UniformDouble(double a, double b) { return PRNG::Instance()->uniformDouble(a,b); }
inline double UniformDouble() { return PRNG::Instance()->uniformDouble(); }
inline double UniformDouble_0() { return PRNG::Instance()->uniformDouble_0(); }
inline double UniformDouble_1() { return PRNG::Instance()->uniformDouble_1(); }
inline double UniformDouble_01() { return PRNG::Instance()->uniformDouble_01(); }
inline unsigned int UniformInt(unsigned int a, unsigned int b) { return PRNG::Instance()->uniformInt(a,b); }

#endif

