#ifndef __INITIALTRANSIENT_H__
#define __INITIALTRANSIENT_H__

class Estimation;


//! Erkennung der initialen transienten Phase.
/*!
 * Diese Klasse realisiert die automatische Erkennung der initialen transienten 
 * Phase bei der Auswertung von Simulationsdaten in CINSim. Es erfolgt ein zweistufiger
 * Test, bei dem zunaechst eine Heuristik zur Bestimmung einer ersten Abschaetzung und 
 * anschliessend der Schruben-Test angewendet wird. Die Heuristik betrachtet das Kreuzen 
 * aufeinander folgender Messwerte bezueglich des aktuellen Mittelwertes aller bisher 
 * gesammelter Werte.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class InitialTransient
{
public:

	//! Moegliche Zustaende.
	enum DetectState{ HEURISTIC, TEST, FINISHED, FAILED };
	
	//! Konstruktor.
	InitialTransient();

	//! Destruktor.
	~InitialTransient();
	
	//! Messwert an Erkennungsroutine uebergeben.
	/*!
	 * Bei der Uebergabe eines neuen Messwertes wird je nach Zustand entweder
	 * die Heuristik oder der Schruben-Test aktualisiert und ggf. der Zustand
	 * neu gesetzt. Ist die Erkennung beendet oder fehlgeschlagen, so wird keine 
	 * Aktion mehr durchgefuehrt.
	 *
	 * @return Zustand nach Hinzufuegen des Messwertes.
	 */
	inline int advance(double value)
	{
		return (this->*advanceInternal)(value);
	}
	
	//! Zustand der Erkennung.
	/*!
	 * Die Erkennung der initialen transienten Phase erfolgt in zwei
	 * Stufen. Zunaechst wird eine Heuristik zur Bestimmung einer ersten
	 * Abschaetzung, dann der Schruben-Test angewendet. Wird dieser mit 
	 * einer positiven Entscheidung beendet, so ist das Ende der initialen 
	 * transienten Phase erreicht. Ein Fehler tritt auf wenn die maximale 
	 * Anzahl zu untersuchender Werte erreicht wird.
	 *
	 * @see DetectState
	 * @return Identifier fuer aktuellen Zustand.
	 */
	int state() 
	{ 
		return _state; 
	}

	//! Anzahl verworfener Werte der transienten Phase.
	/*!
	 * Diese Methode liefert die Anzahl der bereits verworfenen Messwerte
	 * der transienten Phase. Wurde die Erkennung erfolgreich beendet, so
	 * entspricht der Wert der Laenge der initialen transienten Phase.
	 *
	 * @see State
	 * @return Anzahl verworfener Werte.
	 */
	unsigned int discarded() 
	{ 
		return _k; 
	}
	
	//! Daten der Testsequenz an Auswertung weitergeben.
	/*!
	 * Die Daten der Testsequenz des Schruben-Tests gehoeren bei positivem
	 * Abschluss bereits der stationaren Phase an und koennen ueber diesen
	 * Operator der weiteren Auswertung uebergeben werden.
	 *
	 * @see Estimation
	 * @return true, falls Kontrollpunkt erreicht wurde.
	 */
	bool operator>>(Estimation* est) const;

	
protected:

	//! Heuristik.
	int applyHeuristic(double);

	//! Schruben-Test.
	int applyTest(double);

	//! Keine Aktion.
	int applyNop(double);
	
	//! Funktionspointer fuer Erkennungsroutine.
	/*!
	 * Je nach Zustand wird eine Methode zu Bestimmung der initialen 
	 * transienten Phase ueber diesen Pointer angewendet.
	 *
	 * @see applyHeuristic
	 * @see applyTest
	 * @see applyNop
	 * @see advance
	 */
	int (InitialTransient:: *advanceInternal)(double);

	
private:

	//! Datenpuffer.
	double *_buffer;

	//! Summe der Messwerte im Datenpuffer.
	double _bsum;
	
	//! Aktueller Offset im Datenpuffer.
	unsigned int _boffset;
	
	//! Anzahl bereits verworfener Werte.
	unsigned int _k;

	//! Groesse der Testsequenz.
	unsigned int _nt;

	//! Anzahl fehlender Werte fuer Testsequenz.
	unsigned int _delta_nt;

	//! Anzahl zu verwerfender Werte bei Ablehnung durch Schruben-Tests.
	unsigned int _delta_k;
	
	//! Aktueller Zustand der Erkennung.
	DetectState _state;
	
	//! t-Verteilung fuer Schruben-Test.
	double _tdist;
};

#endif
