#ifndef __SPECTRALMEAN_H__
#define __SPECTRALMEAN_H__

#include <estimation.h>

//! Mittelwertabschaetzung durch Spektralanalyse.
/*!
 * Diese Klasse implementiert die Spektralanalyse fuer Mittelwerte nach 
 * Heidelberger/Welch (1981) mit Verwaltung der Simulationsdaten auf konstantem 
 * Speicherplatz (Batching).
 *
 * @see Estimation
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class SpectralMean : public Estimation
{
public:
		
	//! Konstruktor.
	/*!
	 * Die Abschaetzung wird initialisiert.
	 *
	 * @param confidence Das Konfidenzniveau der Abschaetzung.
	 */
	SpectralMean(double confidence);

	//! Destruktor.
	~SpectralMean();
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Diese Ableitung der Klasse Estimation bestimmt Mittelwerte.
	 * 
	 * @see Estimation
	 * @return Estimation::MEAN
	 */
	int estimationType() const
	{
		return Estimation::MEAN;
	}
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Die Methode der Spektral-Analyse umfasst ein spezielles Verfahren
	 * zur Verwaltung der Daten (Batching) auf konstantem Speicherplatz.
	 * diese Methode fuegt ein neues Datum ein. Wird ein gerade bearbeitetes
	 * Batch abgeschlossen, so wird der Mittelwert aktualisiert.
	 *
	 * @see Estimation
	 */
	bool advance(double value);

	//! Implementierung der Klasse Estimation.
	/*!
	 * Die Abstaende zwischen den einzelnen Kontrollpunkten sind zu Beginn 
	 * konstant und werden im spaeteren Verlauf der Abschaetzung, wenn die Groesse 
	 * der Batches den initialen Abstand uebersteigt, nach jedem Batch gesetzt. 
	 * 
	 * @see Estimation
	 */
	bool checkPoint() const;
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Interne Datenstruktur zur Verwaltung der Simulationsdaten wird
	 * geleert und bisherige Abschaetzungen verworfen.
	 * 
	 * @see Estimation
	 */
	void reset();
	
	//! Abschaetzung der Varianz des Mittelwertes.
	/*!
	 * Diese Methode bestimmt eine Abschaetzung der Varianz des Mittelwertes 
	 * der uebergebenen Daten durch Spektralanalyse.
	 * 
	 * @return Varianz der Daten
	 */
	static double Variance(double *data, unsigned int N, double _confidence);
	
protected:

	//! Varianz des Mittelwertes der gesammelter Daten.
	/*!
	 * An Kontrollpunkten wird ueber diese Methode die Varianz anhand
	 * der Spektralanalyse bestimmt.
	 *
	 * @see checkPoint
	 * @return Varianz der gesammelten Daten.
	 */
	double estimateVariance(const double* data, unsigned int n) const;
	
	//! Aktuell bearbeiteter Batch.
	unsigned int _offset;

	//! Aktuelle Groesse der einzelnen Batches.
	unsigned int _batchsize;
	
	//! Fehlende Messwerte fuer aktuellen Batch.
	unsigned int _batchcount;
	
	//! Array mit Batchmeans.
	double* _buffer;

	//! Groesse des Arrays mit Batchmeans.
	unsigned int _buffersize;
	
	//! Frequenzen.
	double* _fj;
	
	//! T(alpha,kappa).
	double _tdist;
};

#endif
