#ifndef __PACKET_H__
#define __PACKET_H__

#include <flit.h>
#include <utility/bitmask.h>
#include <bits/wordsize.h>


//! Die Pakete zur Nachrichtenuebertragung.
/*!
 * Pakete werden als eine Liste von Paketfragmenten, den Flits realisiert. 
 * Das erste Flit der Liste ist dabei immer der Head, das letzte Flit der der 
 * Tail. Aller weiteren Flits sind Datenflits. In einem Takt kann genau ein
 * Flit ueber eine Leitung bewegt werden. Je nach Switching-Strategie muss
 * damit ein Verbindung zwischen zwei Puffern solange aufrecht erhalten werden
 * bis der Tail bewegt wurde.
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin>
 */
class Packet : public Frame
{

// Setzen von ownedflits
friend void Flit::setOwner(Packet*);
	
public:

	//! Standard-Konstruktor.
	/*!
	 * Der Aufruf des Konstruktors erzeugt ein initiales Paket der Grosse 0,
	 * da Pakete und Flits abstrakt. Aufruf von Packet::resize erforderlich.
	 */
	Packet();

	//! Copy-Konstruktor.
	/*!
	 * Der Aufruf des Copy-Konstruktors erzeugt eine flache Kopie und setzt 
	 * den Referenz-Zaehler auf Initialwert 0.
	 *
	 * @param packet Vorlage fuer neues Paket.
	 */
	Packet(const Packet& packet);
	
	//! Paket klonen.
	/*!
	 * Pakete werden vor dem Versenden vom Senderegister eines Puffers fuer 
	 * jeden Empfaenger kopiert. Diese Methode erstellt dafuer eine Kopie 
	 * dieses Paketes. Die zugehoerigen Flits koennen ebenfalls kopiert werden 
	 * (tiefe Kopie) oder zunaechst erst einmal uebernommen werden (flache
	 * Kopie).
	 * 
	 * @see RegisterContainer
	 * 
	 * @param deep Flag fuer tiefe Kopie.
	 * @return Kopie dieses Paketes.
	 */
	virtual Packet* clone(bool deep=false) const = 0;
	
	//! Flache Kopie in tiefe Kopie umwandeln.
	/*!
	 * Soll eine tiefe Kopie eines Paketes erstellt werden, so koennen
	 * alle (noch) nicht kopierten Flits durch Aufruf dieser Methode kopiert
	 * werden.
	 */
	inline void cloneFlits()
	{
		for(unsigned int i=ownedflits; i<size(); i++)
		{
			(*this)[i] = (*this)[i]->clone();
			(*this)[i]->owner = this;
		}
		ownedflits = size();
	}

	/*!
	 * String-Repraesentation.
	 *
	 * @return String-Repraesentation dieses Paketes.
	 */
	virtual std::string toString() const;
	
	//! Zugriff auf die maskierten Ziele des Paketes.
	/*!
	 * Die Ziele an, die ein Paket versendet werden soll, werden in
	 * Form einer Bitmaske repraesentiert. Diese Bitmaske wird mit
	 * der Bitmaske von Netzkomponenten verglichen, um so einen Weg
	 * duch eine Netzkonfiguration zu bestimmen.
	 *
	 * @see Bitmask
	 * @see Component
	 *
	 * @return Die Bitmaske dieses Paketes.
	 */
	inline const Bitmask& getBitmask() const 
	{ 
		return targets; 
	}
	
	/*! 
	 * Setzen der Bitmaske eines Paketes.
	 * 
	 * @param bm Die neue Bitmaske dieses Paketes.
	 */
	inline void setBitmask(const Bitmask& bm) 
	{ 
		targets = bm; 
	}
    
	/*!
	 * Setzen eins Bits der Bitmaske eines Pakets.
	 *
	 * @param target_nr Das zu setztende Bit (Nummer eines Zieles).
	 */
	inline void setTarget(unsigned int target_nr) 
	{ 
		targets.set(target_nr); 
	}
	
	//! Paket referenzieren.
	/*!
	 * Das Referenzieren eines Paketes soll verhindern, dass ein Paket 
	 * unerwartet bzw. zu frueh geloescht wird, wenn es von einer Stufe
	 * zur naechsten versendet wird.
	 *
	 * @see dereference
	 * @see release
	 */
	inline void reference() 
	{ 
		refcount++; 
	}
		
	/*!
	 * Referenz auf Paket aufheben.
	 *
	 * @see reference
	 * @see release
	 */
	inline void dereference() 
	{ 
		refcount = refcount>0 ? refcount-1 : 0; 
	} 
	
	//! Ein Paket freigeben.
	/*!
	 * Die Referenz auf das Paket wird aufgehoben. Sollte es sich um die
	 * letzte Referenz handeln, so wird der Destruktor aufgerufen.
	 *
	 * @see reference
	 * @see dereference
	 */
	inline void release()
	{
		if(refcount>1) refcount--; else delete this;
	}
		
	//! Globale Anzahl von Paketen der Simulation.
	/*!
	 * Pakete werden von Quellen generiert, vor dem Weitersenden kopiert
	 * und nach dem Eintreffen an den Zielen geloescht. Intern wird bei
	 * Verwendung der Konstruktoren und des Destruktors entsprechend ein
	 * globaler Zaehler modifiziert. Unter der Annahme, dass immer nur 
	 * eine Netzkonfiguration aktiv ist, entspricht dieser Zaehlerstand 
	 * der Anzahl der Pakete in dieser Netzkonfiguration.
	 *
	 * @see Simulation
	 * @see Configuration
	 * 
	 * @return Anzahl existierender Pakete in aktiver Netzkonfiguration.
	 */
	inline static unsigned int Existing()
	{
		return _packetCount;
	}
	
	//! Die Nummer des Paketes.
	unsigned int number;
    
	//! Der Geburtstakt des Paketes.
	unsigned int createTick;
	
	//! Die Prioritaet eines Pakets.
	int priority;

	//! Der Takt, in dem das Paket sein Ziel erreicht haben muss.
	unsigned int deadline;

	//! Index der Paketquelle.
	unsigned int sourcenr;
		
protected:

	//! Destruktor.
	/*!
	 * Der Aufruf des Destruktors zerstoert alle zu diesem Paket gehoerenden
	 * Flits und gibt deren Speicher frei. Der Aufruf erfolgt normalerweise
	 * ueber die Methode Packet::release.
	 */
	virtual ~Packet();

	//! Erzuegen eines passenden Flits.
	/*!
	 * Jedes Switching-Verfahren definiert eigene Arten von Paketen und
	 * damit auch eigene Arten von Flits. Wird ein Paket erstellt, so
	 * werden ueber diese Methode die passenden Flist generiert.
	 * 
	 * @param offset Die Position des Flits im Paket.
	 * @param maxoffset Die Position des letzten Flits (Tail).
	 * @return Passendes Flit fuer dieses Paket.
	 */
	virtual Flit* createFlit(unsigned int offset, unsigned int maxoffset) = 0;

	//! Packetgroesse veraendern.
	/*!
	 * Diese Methode modifiziert die Groesse eines Paketes. Die Groesse
	 * beschreibt die Anzahl (gleichgrosser) Flits, aus denen das Paket
	 * besteht. Es ist nur zulaessig die Anzahl von der Flits eines Paketes
	 * zu erhoehen. Die initiale Groesse eines Paketes ist 0. Der Aufruf
	 * modifiziert Paket zu Mindestgroesse 1.
	 * 
	 * @param size Groesse.
	 * @return true, fall neue Groesse zulaessig und Paket angepasst. 
	 */
	bool resize(unsigned int size);

private:

	//! Aufsteigende Nummern fuer neue Pakete.
	static unsigned int _packetNumber;
	
	//! Zaehler fuer existierende Nachrichtenpakete.
	static unsigned int _packetCount;
	
	//! Die maskierten Ziele des Paketes.
	Bitmask targets;

	//! Refernz-Zaehler fuer Register-Container.
	unsigned int refcount;

	//! Zaehler fuer eigene Flits.
	unsigned int ownedflits;
};

#endif
