#ifndef __PACKETFORWARDING_H__
#define __PACKETFORWARDING_H__

#include <utility/xmlelement.h>
#include <utility/factory.h>
#include <triggernode.h>
#include <wirematrixmodifier.h>

class Routing;
class Switching;

//! Schaltmatrix-Manipulator fr zulaessige Paketbewegung.
/*!
 * Diese Klasse sorgt fuer die Anpassung der Schaltmatrix eines Routers 
 * bezueglich des verfuegbaren Platzes am Ausgang eines Routers. Dazu wird 
 * das Bit IO_FIT eines Matrixeintrages gesetzt, wenn der Netzknoten am 
 * zugehoerigen Ausgang ein zu vermittelndes Paket eines Einganges 
 * aufnehmen kann.
 *
 * @see WireMatrixEntry
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ForwardingModifier : public WireMatrixModifier
{
public:

	//! Implementierung der abstrakten Basisklasse WireMatrixModifier.
	/*!
	 * Wenn der nachfolgende Puffer an diesem Ausgang das anliegende 
	 * Paket annehmen kann, so wird das Bit IO_FIT gesetzt.
	 */
	virtual void modify(WireMatrixIterator*);
};

//! Basis-Klasse zur Paketbewegung waehrend einer Simulation.
/*!
 * Diese Klasse stellt die Basisklasse fuer eine Koroutine der 
 * Simulation dar, die fuer die Bewegung der Pakete im Netz waehrend 
 * eines Taktes verantwortlich ist. Eine Koroutine beschreibt jeweils
 * die zum Einsatz kommende Backpressure-Strategie (lokales, globales
 * Backpressure oder z.B. Paketverlust) und verwaltet das Routing- und
 * Switching-Verfahren der Simulation.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class PacketForwarding : public XMLElement, public FactoryElement
{
public:
	
	//! Standard-Konstruktor.
	/*!
	 * Der Konstruktor bekommt als Argument einen Ausschnitt des
	 * eingelesenen XML-Dokuments in Form eines Element-Knotens
	 * des vom Parser erstellten DOM-Baumes. Dieser Knoten (oder
	 * untergeordnete Knoten) liefert Informationen zu Routing
	 * und Switching der Simulation.
	 * 
	 * @param scope XML-Element mit Spezifikation der Paketbewegung.
	 */
	PacketForwarding(const DOMElement* scope=NULL) 
		: XMLElement(scope), _forwarding(NULL), _routing(NULL), _switching(NULL) 
		{
			// Initialisiere Switching-Flags
			isPCT=false;
			isVCT=false;
		}
	
	//! Destruktor.
	virtual ~PacketForwarding();
	
	//! Implementierung der abstrakten Basisklasse XMLElement.
	/*!
	 * Die Komponenten der Koroutine werden anhand der Informationen 
	 * aus dem sichtbaren Teiles des XML-Dokumentes erstellt und
	 * initialisiert.
	 * 
	 * @param scope XML-Element mit Spezifikation der Paketbewegung.
	 */
	bool init(const DOMElement* scope=NULL);
	
	//! Start der Paketbewegung.
	/*!
	 * Diese Methode stoesst die Paketbewegungen in einem Netz
	 * an. Der Aufruf erfolgt jeweils einmal in jedem Takt und
	 * sorgt dafuer, das alle im Netz befindlichen Pakete, je nach
	 * verfolgter Strategie, vermittelt werden oder in ihrem 
	 * jeweiligen Puffer-Knoten verbleiben.  
	 */
	virtual void start() = 0;
	
	//! Das Routing-Verfahren der Simulation.
	/*!
	 * Das Routing der Simulation legt fest, welchen Weg ein
	 * Paket von der Quelle zum Ziel in einem Netz benutzt. Diese
	 * Methode liefert einen entsprechenden Manipulator fuer
	 * die Router des Netzes, der fuer die an den Eingaengen eines
	 * Routers wartenden Pakete, die moeglichen Ausgaenge bestimmt.
	 */
	inline Routing* routing() const
	{
		return _routing;
	}

	//! Das Switching-Verfahren der Simulation.
	/*!
	 * Das Switching der Simulation sorgt fuer die Bewegung
	 * von Paketen bzw. von Paketfragmenten, indem an Routern
	 * beispielsweise Schaltungen reserviert oder Ein-/Ausgaenge
	 * deaktiviert werden. Diese Methode liefert einen entsprechenden 
	 * Manipulator fuer die Router des Netzes, der zum Schalten eines 
	 * Routers zum Einsatz kommt.
	 */
	inline Switching* switching() const
	{
		return _switching;
	}

	//! Der Backpressure-Mechanismus der Simulation.
	/*!
	 * Ein Koroutine beschreibt eine Backpressure-Strategie (oder
	 * z.B. Paketverlust). Fuer einen Router ist es wichtig zu bestimmen,
	 * ob ein Paket, von einem Knoten an einem Ausgang aufgenommen
	 * werden kann. Je nach Strategie, kann diese Information
	 * jederzeit bestimmbar oder aber u.U. noch nicht bestimmbar sein. 
	 */
	inline WireMatrixModifier* forwarding() const
	{
		return _forwarding;
	}

	bool isPCT;
	bool isVCT;

	
protected:

	//! Initialisierung des Modifiers fuer Paketbewegung.
	/*!
	 * Die Impkementierung einer Koroutine muss ein Verfahren, in Form
	 * eines Manipulators fuer Schaltmartitzen, bereitstellen, welches
	 * bestimmt, ob ein Paket/Flit bewegt werden kann. Diese Methode
	 * wird von init einmalig aufgerufen und kann auch verwendet werden,
	 * um Initialisierungsdetails, die nicht von init erschlagen werden,
	 * zu realisieren.
	 * 
	 * @see ForwardingModifier
	 * @see init
	 * 
	 * return true, falls Initialisierung erfolgreich.
	 */
	virtual ForwardingModifier* initForwarding() = 0;

	
private:

	//! Manipulator fuer zulaessige Paketbewegung.
	ForwardingModifier* _forwarding;

	//! Routing-Verfahren der Simulation. 
	Routing* _routing;

	//! Switching-Verfahren der Simulation.
	Switching* _switching;
};

#endif
