#ifndef __ROUTER_H__
#define __ROUTER_H__

#include <node.h>
#include <arbitercontroller.h>
#include <wirematrix.h>


//! Manipulator zum Zuruecksetzen alter Schaltzustaende.
/*!
 * Diese Klasse beschreibt die Modifikationen der Schaltmatrix, zu Beginn eines
 * neuen Taktes. Die Vorbedingung ist per default stets erfuellt.
 *
 * @see WireMatrixModifier
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class PreSwitching : public WireMatrixModifier
{
public:

	//! Modifikation.
	/*!
	 * Alte Schaltzustaende werden entfernt, d.h. die meisten Eintraege in den
	 * Bitmasken auf 0 gesetzt. Ausnahmen sind I_ACTIVE und O_ACTIVE und fuer den
	 * Fall einer Reservierung entsprechende IO_VALID, I_SWITCHED und O_SWITCHED.
	 * Eine Reservierung wird durch I_HOLD und O_HOLD angezeigt.
	 */
	void modify(WireMatrixIterator*);
};


//! Die Router eines Verbindungsnetzes.
/*!
 * Diese Klasse beschreibt Router (Switches) eines Verbindungsnetzes. Router sind
 * Knoten eines Graphen mit beliebiger Anzahl ein- und ausgehender (gerichteter)
 * Kanten, den Leitungen.
 *
 * @see Node
 * 
 * Router vermitteln Nachrichtenpakete an ihren eingehenden Leitungen an ausgehende 
 * Leitungen, entsprechend den Zielen, an die ein Paket gesendet werden soll. Dazu
 * wird der Head eines Paketes auf die in Form einer Bitmaske maskierten Ziele
 * hin untersucht und die moeglichen Ausgaenge in einer Schaltmatrix markiert.
 * Wird ein Augang durch die Koroutine aktiviert, so wird unter den markierten
 * Eingaengen durch ein Scheduling ein Eingang gewaehlt und auf den aktivierten 
 * Ausgang geschaltet. Ein solche Schaltung wird in der Schaltmatrix markiert und
 * bleibt bestehen, bis der Tail eines Paketes ueber diese Verbindung versendet
 * wurde.
 * 
 * @see Packet
 * @see WireMatrix
 * @see Arbiter
 * @see ArbiterController
 * 
 * Ein Router ist zwar prinzipiell in der Lage Messungen durchzufuehren, aber
 * derzeit sind keine Messungen definiert.
 *
 * @see Analysable
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Router : public Node
{
public:

	//! Konstruktur.
	/*!
	 * Der Konstruktor initialisert die Schaltmatrix mit den Referenzen auf
	 * die Vektoren der Ein- und Ausgaenge und setzt den Pointer fuer den 
	 * Arbitercontroller auf NULL. In seiner Eigenschaft als XMLElement kann 
	 * auch ein zugehoeriges DOMElement fuer den Router spezifiziert werden.
	 *
	 * @see XMLElement
	 *
	 * @param scope Zugehoeriges DOMElement mit Beschreibung dieses Routers.
	 */
	Router(const DOMElement* scope=NULL) 
		: _seen(0), _wm(In(),Out()), _ac(NULL) { setScope(scope); }

	//! Implementierung der abstrakten Basisklasse XMLElement.
	/*!
	 * Der Aufruf dieser Methode sorgt dafuer, dass eine Instanz der Klasse
	 * ArbiterController ueber die Factory generiert wird und fuer jeden
	 * Ausgang eine Instanz der Klasse Arbiter. 
	 * 
	 * @see XMLElement
	 * @see Arbiter
	 * @see ArbiterController
	 * @return true, falls Initialisierung vollstaendig.
	 */
	virtual bool init(const DOMElement* scope=NULL);

	//! Implementierung der abstrakten Basisklasse Analysable.
	/*!
	 * Es sind derzeit keine Messungen fuer Router definiert, daher wird
	 * stets der Wert 0.0 zurueckgeliefert.
	 * 
	 * @see Analysable
	 * @return 0.0
	 */
	virtual double Measure(TypeInfo) { return 0.0; };	
	
	/*!
	 * Implementierung der abstrakten Basisklasse Component.
	 *
	 * @see Component
	 * @return Component::ROUTER.
	 */
	virtual const int rtti() const { return Component::ROUTER; }

	/*!
	 * Implementierung der abstrakten Basisklasse Component.
	 * 
	 * @see Component
	 * @return "Router".
	 */
	virtual const std::string Name() const 
	{
		return std::string ("Router");
	}

	//! Implementierung der abstrakten Basisklasse Component.
	/*!
	 * Die Bitmaske eines Routers ergibt sich aus den Bitmasken aller
	 * nachfolgenden Elemente in Senderichtung.
	 * 
	 * @see Component
	 */
	virtual const Bitmask& generateBitmask();

	//! Implementierung der abstrakten Basisklasse Component.
	/*!
	 * Die Eintrage der Schaltmatrix werden zurueckgesetzt und alle Schaltungen
	 * auf ausgehende Leitung freigegeben.
	 *
	 * @see Component
	 */
	virtual void reset();
	
	//! Implementierung der abstrakten Basisklasse Node.
	/*! 
	 * Soll ein Router ein Paket auf eine seiner ausgehenden Leitungen legen, so 
	 * muss zunaechst aus der Menge von Kandidaten, eingehende Leitungen mit 
	 * angelegten Paketen, ein Kandidat ueber ein Scheduling ermittelt werden. Ist 
	 * dies der Fall, so wird Node::SUCCESS zurueckgeliefert. Wollen keine Pakete
	 * diesen Ausgang benutzen, ist das Ergebnis Node::NOPACKET, koennen keine
	 * Pakete diesen Ausgang benutzen Node::DISMISS. Fuer den Fall, dass noch 
	 * kein Kandidat abschliessend ermittelt werden kann, wird Node::WAIT 
	 * zurueckgegeben.
	 * 
	 * @see Node
	 */
	virtual int putPacketOnLine(Wire*, bool&);

	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Ein Router kann keine Pakete empfangen und sollte auch nicht dazu 
	 * aufgefordert werden.
	 *
	 * @see Node
	 * @return Node::ERROR. 
	 */
	virtual int takePacketFromLine(Wire*, bool&) 
	{ 
		return Node::ERROR; 
	}

	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Wird eine eingehende Leitung eines Routers aktiviert, d.h. ein Vorgaenger
	 * im Netzgraphen aktiviert, so werden in der internen Schaltmatrix die alten
	 * Zustaende entfernt und, sofern ein (neues) Paket anliegt, ein Routing 
	 * durchgefuehrt, wodurch die moeglichen ausgehenden Leitungen fuer dieses 
	 * Paket markiert werden. Dazu liefert das zweite Argument, die zu verwendende
	 * Routing-Logik.
	 *
	 * @see Node
	 * @return Node::SUCCESS
	 */
	virtual int lineActivated(Wire*, ModifierList*, 
							SignedTNodeList* =NULL, TNodeList::iterator=NLITNULL);
	
	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Der Aufruf dieser Methode fordert einen Router dazu auf, die aufgrund seiner
	 * erzeugten Schaltung fuer den uebergebenen Ausgang aufgeloesten Knoten zu
	 * bestimmen. Das sind die Knoten, deren Sende-Status fuer diesen Takt geklaert
	 * ist, also der Gewinner und alle Verlierer, die keine Chance auf einen weiteren
	 * Ausgang besitzen. Wurde ein Paket vollstaendig ueber eine ausgehende 
	 * Leitung vermittelt, wird die Schaltung aufgehoben, wozu das zweite Argument
	 * verwendet wird.
	 * 
	 * @see Node
	 * @return Node::SUCCESS
	 */
	virtual int lineFinished(Wire*, ModifierList*, 
							SignedTNodeList* =NULL, TNodeList::iterator=NLITNULL);

	
protected:

	//! Implementierung der Basisklasse Node.
	/*!
	 * Wird eine eingehende Leitung fuer diesen Router hinzugefuegt, so muss die
	 * interne Schaltmatrix entsprechend aktualisiert werden.
	 *
	 * @see Node
	 */
	virtual unsigned int addInWire(Wire*);
	
	//! Implementierung der Basisklasse Node.
	/*!
	 * Wird eine ausgehende Leitung fuer diesen Router hinzugefuegt, so muss die
	 * interne Schaltmatrix entsprechend aktulalisiert werden.
	 *
	 * @see Node
	 */
	virtual unsigned int addOutWire(Wire*);
	
	
private:
	
	//! Flag fuer rekursive Generierung von Bitmasken.
	bool _seen;

	//! Die interne Schaltmatrix (Inwires X Outwires).
	/*!
	 * Die Schaltmatrix eines Routers verwaltet alle Informationen zu
	 * Schaltzustaenden.
	 * 
	 * @see WireMatrix
	 */
	WireMatrix _wm;

	//! Arbiter-Controller. 
	/*!
	 * Steuert die Arbiter der Ausgaenge eines Routers und erlaubt dadurch
	 * (theoretisch) ein koordiniertes Scheduling mehrerer Arbiter.
	 *
	 * @see ArbiterController.
	 */
	ArbiterController* _ac;

	//! Modifier zum entfernen alter Schaltzustaende.
	/*!
	 * Wird eine eingehende Leitung eines Routers durch die Koroutine 
	 * aktiviert, so muessen zunaechst alte Zustaende aus dem letzten
	 * Takt entfernt werden.
	 * 
	 * @see PreSwitching
	 */
	PreSwitching _preswitch;
};

#endif
