#ifndef __ROUTING_H__
#define __ROUTING_H__

#include <wirematrixmodifier.h>
#include <utility/factory.h>

class WireMatrixIterator;
class Wire;
class Packet;
class Bitmask;

//! Abstrakte Basisklasse fuer Routing-Verfahren.
/*!
 * Ein Routing-Verfahren bestimmt den Weg eines Paketes in einem Verbindungsnetz 
 * von seiner Quelle zu seinem Ziel bzw. seinen Zielen. An einem Router muss 
 * jeweils entschieden werden, welche der moegichen Ausgaenge, einen Weg zu 
 * mindestens einem Ziel darstellen. Diese Informationen werden in der Schaltmatrix 
 * eines Routers verwaltet. Ein Routing-Verfahren modifiziert die Schaltmatrix 
 * eines Routers derart, dass die moeglichen Ausgaenge eines Paketes an einem 
 * Eingang sichtbar werden.
 *
 * @see WireMatrixModifier
 * @see WireMatrixIterator
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Routing : public WireMatrixModifier, public FactoryElement 
{	
public:	

	//! Implementierung des Interfaces WireMatrixModifier.
	/*!
	 * Die Vorraussetzung fuer die Durchfuerhung eines Routings sind ein 
	 * anliegendes Paket (I_PACKET) und noch keine vorhandene
	 * Routingentscheidung (IO_VALID).
	 * 
	 * @see WireMatrixEntry
	 * @param it Iterator zu Eintrag einer Schaltmatrix eines Routers.
	 * @return WireMatrixModifier::PASSED oder WireMatrixModifier::FAILED.
	 */
	int precondition(WireMatrixIterator* it) const;
	
	//! Implementierung des Interfaces WireMatrixModifier.
	/*!
	 * Kann ein Routing durchgefuehrt werden, so setzt diese
	 * Methode das Flag IO_VALID, falls das zu vemittelnde
	 * Paket den betrachteten Ausgang benutzen kann.
	 *
	 * @see WireMatrixEntry
	 * @param it Iterator zu Eintrag einer Schaltmatrix eines Routers.
	 */
	void modify(WireMatrixIterator* it);
	
	//! Initialisierung.
	/*!
	 * Wird eine Instanz eines Routingverfahrens erstellt, so kann ueber 
	 * diese Methode einmalig eine Initialisierung durchgefuehrt werden.
	 *
	 * @return true, falls Initialisierung fehlerfrei.
	 */
	virtual bool init() = 0;
			
// protected:
// protected ist auskommentiert, da die Bitmasken direkt von simulation.cpp verwendet werden
// beim Kopieren der Pakete in das neue Netz.
	//! Routing-Logik.
	/*!
	 * Diese Methode soll einen Test implementieren, der festlegt, ob ein 
	 * Paket auf dem Weg zu seinem Ziel den uebergebenen Ausgang eines Routers
	 * benutzen kann. Alle Routing-Verfahren implementieren diese Methode.
	 * Das Ergebnis ist entweder eine Bitmaske mit ueber den betrachteten
	 * Ausgang erreichbaren Zielen des des Paketes oder NULL, falls kein
	 * angestrebtes Ziel ueber den Ausgang erreicht werden kann.
	 * 
	 * @param ptr Das anliegende Paket (zu einem Head-Flit).
	 * @param remaining Offene Ziele des anliegenden Paketes.
	 * @param output Ein zu untersuchender Ausgang.
	 * @return Nichtleere Bitmaske, falls Paket ueber spezifizierten Ausgang 
	 * zu offenen Zielen versendet werden kann, oder NULL.
	 */
	virtual Bitmask* validOutput(const Packet* ptr, const Bitmask* remaining, const Wire* output) = 0;	
};

#endif

