#ifndef __SEQUENCER_H__
#define __SEQUENCER_H__

#include <xmlload.h>
#include <environment/analysis.h>

#include <vector>

//! Simulationssequenz.
/*!
 * Der Sequenzer kapselt die Simulation und erlaubt deren Wiederholung
 * oder eine Sequenz mit variierten (numerischen) Parametern. End- und 
 * Zwischenergebnisse der einzelnen Laeufe werden verwaltet und koennen 
 * zur Laufzeit ausgegeben werden, um den Fortschritt zu verfolgen.
 *
 * @see Simulation
 * @see Environment
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Sequencer : public XmlLoad
{

friend int main(int,char**);
	
public:

	//! Ein Parameter fr Simulationssequenzen.
	struct Parameter 
	{
		//! Der Startwert.
		double value;
		//! Der Offset.
		double increment;
		//! name =^ id
		std::string name;
	};

	//! Das Setup einer Simulationssequenz
	struct Setup
	{
		//! Typ.
		std::string stype;
		//! Name
		std::string sname;
		//! Beginn der transienten Phase
		unsigned int tstart;		
		//! Ende der transienten Phase
		unsigned int tend;
		//! Laenge der Sequenz.
		unsigned int nofsim;
		//! Anzahl der Messungen pro Lauf.
		unsigned int nofobs;
	};
	
	//! Globale Instanz des Sequencers.
	/*!
	 * Es existiert stets genau eine Instanz der Klasse Sequencer, auf die
	 * ueber diese Methode zugegriffen werden kann, um aktuelle Informationen 
	 * zum Status zu erlangen.
	 *
	 * @return Globale Instanz der Klasse Sequencer.
	 */
	inline static const Sequencer* Instance()
	{
		return _instance;
	}
	
	//! Signal-Handling.
	/*!
	 * Waehrend einer Simuationssequenz ist die Ausgabe von Zwischenergebnissen
	 * durch Betaetigung von ctrl-c moeglich und der Abbruch durch zweimaliges 
	 * Betaetigen von ctrl-c innerhalb von 2 Sekunden. Das benutzerdefinierte
	 * Signal SIGUSR2 kann ebenfalls zur Ausgabe von Ergebnissen verwendet werden.
	 *
	 * @param signr Signalnummer.
	 */
	static void SignalHandler(int signr);

	//! Laden einer Simulationssequenz
	/*!
	 * Diese Methode erstellt eine Simulationssequenz aus einer XML-Beschreibung.
	 * Dazu wird die Methode der Basisklasse aufgerufen, alle Parameter der Sequenz
	 * im Environment initialisiert und anschliessend die Simulation geladen.
	 *
	 * @see Environment
	 * @see Simulation
	 * @see XmlLoad::load()
	 */
	void load();

	//! Starten der Simulationssequenz.
	/*!
	 * Diese Methode startet eine Simulationssequenz, bei der eine Simulation
	 * mit verschiedenen Parametern ausgef\"uhrt werden kann. Jeder Lauf kann
	 * dabei mehrfach wiederholt werden.
	 * 
	 * @pre XmlSequencer::load
	 * @see Simulation
	 */
	void start();
		
	//! Ausgabe der Ergebnisse einer Simulationssequenz.
	/*!
	 * Die aktuellen Ergebnisse werden je nach Konfigurationen auf stdout, in
	 * eine Datei und/oder als Plot ausgegeben.
	 */
	void output() const;
	
	//! Ausgabe von Zwischenergebnissen einer Simulationssequenz.
	/*!
	 * Die aktuellen Zwischenergebnisse werden auf stdout ausgegeben.
	 */
	void intermediate() const;
	
	//! Setup-Informationen.
	/*!
	 * TODO: Remove??
	 */
	inline const Sequencer::Setup& setup() const
	{
		return _instance->_setup;
	}
	
	//! Simulationslauf.
	/*!
	 * Der Index des aktuellen Simulationslaufes in Sequenz. Erster
	 * Lauf hat Index 0.
	 *
	 * @return Index des aktuellen Simulationslaufes.
	 */
	inline unsigned int Run() const
	{
		return _instance->_simulation;
	}

	//! Wiederholung des Simulationslaufes.
	/*!
	 * Die Wiederholung eines Simulationslaufes wird durch Rueckgabewert
	 * groesser 0 angezeigt, erster Lauf hat Rueckgabewert 0.
	 *
	 * return Wiederholung des Simulationslaufes.
	 */
	inline unsigned int Repetition() const
	{
		return _instance->_repetition;
	}

protected:
	
	//! Geschuetzter Konstruktor.
	Sequencer() : _simulation(0), _repetition(0), _results(0) {}

	//! Geschuetzter Destruktor.
	/*!
	 * Datenstruktur mit Ergebnissen wird freigegeben.
	 */
	virtual ~Sequencer();
	
	//! Laden einer Simulationssequenz aus XML-Beschreibung.
	/*!
	 *
	 * @pre Instance() != NULL
	 * @see XmlSequencer::load
	 */
	static void Load()
	{
		_instance->load();
	}
	
	//! Start einer Simulationssequenz.
	/*!
	 *
	 * @pre Instance() != NULL
	 * @pre Sequence::Load()
	 * @see start
	 */
	inline static void Start()
	{
		_instance->start();
	}

	//! Die Parameter einer Simulationssequenz.
	std::vector<Sequencer::Parameter> _parameters;
	
	//! Das Setup einer Simulationssequenz
	Sequencer::Setup _setup;
	
	//! Zaehler fuer Simulationslauf.
	unsigned int _simulation;

	//! Zaehler fuer Wiederholung eines Simulationslaufes.
	unsigned int _repetition;
	
	//! Simulationsergebnisse.
	Analysis::Result** _results;
	
	//! Simulationssequenz.
	static Sequencer* _instance;
};

#endif
