#ifndef __SIMULATION_H__
#define __SIMULATION_H__

#include <xmlload.h>

#include <vector>
#include <cassert>


class Configuration;
class DLHandler;

//! Die Simulation.
/*!
 * Instanzen dieser Klasse beschreiben die Simulation eines Verbindungsnetzes
 * oder einer Rekonfgurationsfolge eines Verbindungsnetzes. Eine Simulation
 * umfasst also mindestens eine Netzkonfiguration, die aus einer XML-Beschreibung
 * geladen wird. Exitieren mehrere Konfigurationen, so ist immer genau eine
 * aktiv. Endet deren Laufzeit, so wird auf die nachfolgende Konfiguration
 * umgeschaltet. Die Laufzeit einer Simulation wird von der stochastischen
 * Auswertung der von den einzelnen Konfigurationen durchgefuehrten Messungen
 * bestimmt. Eine Simulation kann entweder stationaere Eigenschaften der 
 * Netzkonfguration(en) bestimmen oder derenen transientes Verhalten untersuchen.
 * 
 * @see Configuration
 * @see XmlLoad
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Simulation : public XmlLoad
{
	
// Load, Info, Start
friend class Sequencer;
friend int main(int,char**);

public:
	
	//! Status-Informationen.
	/*!
	 * Ein Simulation befindet sich entweder im Normalzustand, einer Leerlauf-
	 * oder einer Umschaltphase.
	 */
	enum StateInfo 
	{
		NORMAL,	// Pakete an Quellen generieren 
		FLUSH,	// Leerlaufphase
		SWITCH, // Umschalten
		PREPARE,// Rekonfiguration vorbereiten

		NOSTATE
	};

	//! Simmulationsarten.
	/*!
	 * Ein Simulation kann entweder stationaere Eigenschaften oder transientes
	 * Verhalten untersuchen.
	 */
	enum TypeInfo
	{
		STEADY_STATE,
		TERMINATING,

		NOTYPE
	};
	
	//! Globale Instanz der Simulation.
	/*!
	 * Es existiert stets genau eine Instanz der Klasse Simulation, auf die
	 * ueber diese Methode zugegriffen werden kann, um aktuelle Informationen 
	 * zum Status zu erlangen.
	 *
	 * @return Globale Instanz der Klasse Simulation.
	 */
	inline static const Simulation* Instance()
	{
		return _instance;
	}
		
	//! Destruktor.
	/*!
	 * Die Netzkonfigurationen der Simulation werden freigegeben.
	 *
	 * @see Configuration.
	 */
	virtual ~Simulation();
	    
	//! Starten einer Simulation.
	/*!
	 * Nachdem eine Simulation aus einer XML-Beschreibung erstellt wurde, kann 
	 * sie durch Aufruf dieser Methode gestartet werden. Ein Simulation
	 * wird in der Regel automatisch beendet, wenn die Auswertungen aller 
	 * Messungen der geforderten Konfidenz und Genauigkeit genuegen. Am Ende
	 * einer Simulation sind alle Konfigurationen leer und bereit fuer einen
	 * Neustart im initialen Zustand.
	 */
	void start();
	
	//! Simulation neu laden.
	/*!
	 * In einer Simulationssequenz k\"onnen numerische Parameter von Komponenten
	 * variiert werden, die Struktur des Simulationsaufbaus bleibt aber bestehen.
	 * Diese Methode sorgt dafuer, dass die Netzwerkkomponenten aller Konfigurationen
	 * anhand ihrer XML-Beschreibung neu initialsiert werden, veraenderte Parameter
	 * koennen dabei vom Environment bezogen werden.
	 *
	 * @see Environment
	 * @see Sequencer
	 */
	void reload();
	
	//! Die aktive Konfiguration der Simulation.
	/*!
	 * Waehrend einer Simulation ist stets genau eine Netzkonfiguration 
	 * aktiv. Eine aktive Konfiguration wird, ausser in der Umschaltphase
	 * einer Rekonfiguration, einmal pro Takt angestossen, fuehrt 
	 * Paketbewegungen aus und liefert Messwerte an die stochastsche 
	 * Auswertung.
	 *
	 * @see Configuration
	 */
	inline const Configuration* Active() const
	{
		assert(_activeConfig != NULL);
		
		return _activeConfig;
	}
	
	//! Der Simulationstakt.
	/*!
	 * Stationaere Simulationen beginnen in Takt 0 und fuehren sukzessive
	 * weitere Takte aus bis die durchgefuehrten Messungen einen
	 * stationaeren Wert erreicht haben. Transiente Simulationen werden
	 * hingegen nur bis zum letzten Messtakt ausgefuehrt und dann neu
	 * gestartet, d.h. in Takt 0 und mit der initialen (leeren) 
	 * Netzkonfiguration.
	 *
	 * @returns Aktueller Takt der Simulation.
	 */
	inline unsigned int Tick() const
	{
		return _globalTick;
	}
	
	/*!
	 * Der Status der Simulation.
	 * 
	 * @see StateInfo
	 * @return Aktueller Zustand der Simulation.
	 */
	inline StateInfo State() const
	{
		return _state;
	}

	/*! 
	 * Die Art der Simulation.
	 *
	 * @see TypeInfo
	 * @return Die Art der Simulation.
	 */
	inline TypeInfo Type() const
	{
		return _type;
	}

	/*! Der DLHandler der aktiven Konfiguration
	 *
	 */
	inline DLHandler* Handler() const
	{
		return _handler;
	}
	
	//! Der erste Messtakt einer Simulation.
	/*!
	 * Transiente Simulationen erlauben Rueckschluesse auf transientes
	 * Verhalten der simulierten Netzkonfiguration(en). Dazu werden
	 * beginnend mit dem ersten Messtakt fuer alle folgenden Takte bis
	 * einschliesslich des letzte Messtaktes und jede Messung eine
	 * stochastische Auswertung duchgefuehrt. Fuer stationaere 
	 * Simulationen ist Takt 0 der erste Messtakt. 
	 *
	 * @see AnalysisEnd
	 * @return Erster Messtakt der Simulation.
	 */
	inline unsigned int AnalysisStart() const
	{
		return _analysisStart;
	}

	//! Der letzte Messtakt einer Simulation.
	/*!
	 * Erreicht eine transiente Simulation den letzten Messtakt, so
	 * wird die Simulation in ihrem initialen Zustand neu gestartet.
	 * Dadurch werden fuer alle Messungen in allen Takten seit dem
	 * ersten Messtakt Werte gesammelt, die eine stochastische Auswertung
	 * erlauben. Fuer stationaere Simulation wird ein maximaler Wert
	 * fuer den letzten Messtakt geliefert.
	 * 
	 * @see AnalysisStart
	 * @return Letzter Messtakt der Simulation.
	 */
	inline unsigned int AnalysisEnd() const
	{
		return _analysisEnd;
	}

	bool show_actions;
	
protected:

	//! Konstruktor.
	/*!
	 * Der Aufruf des Konstruktors setzt Standardwerte fuer die 
	 * Klassenattribute. Die eigentliche Initialisierung erfolgt durch
	 * Implementierung der Klasse XmlLoad.
	 * 
	 * @see XmlSimulation
	 */
    Simulation();
	
	//! Initialisierung der Simulation aus XML-Beschreibung.
	/*!
	 * Die globale Instanz der Klasse Simulation wird durch Aufruf dieser Methode
	 * aus einer spezifizierten XML-Beschreibung initialisiert.
	 *
	 * @see XmlLoad
	 */
	inline static void Load()
	{
		_instance->load();
	}

	//! Reinitialsierung einer Simulation in Simulationssequenz.
	/*!
	 * Soll die Simulation in einer Simulationssequenz neu geladen werden,
	 * so erfolgt der Aufruf ueber diese Methode.
	 * 
	 * @pre Packet::Existing() == 0
	 * @see reload()
	 */
	inline static void Reload()
	{
		_instance->reload();
	}
	
	//! Ausgabe von Informationen zu (initialisierter) Simulation.
	/*!
	 * Nach der Initialisierung der Simulation aus einer XML-Beschreibung
	 * koennen ueber diese Methode Information ausgegeben werden.
	 */
	static void Info();

	//! Start der Simulation.
	/*!
	 * Diese Methode startet die (initialisierte) Simulation.
	 *
	 * @pre Aufruf von Load()
	 * @see start()
	 */
	inline static void Start()
	{
		_instance->start();
	}
	
//
// Setup-Informationen
//
	
	//! Globale Instanz der Simulation.
	static Simulation* _instance;

	//! Konfigurationen der Simulation.
	std::vector<Configuration*> _configurations;

	//! Art der Simulation.
	TypeInfo _type;

	//! Initiale Netzkonfiguration.
	Configuration* _initialConfig;	

	//! Der DLHandler fr die jeweils aktive Konfiguration.
	DLHandler* _handler;
	
	//! Beginn der Messungen fuer stochastischen Analyse.
	unsigned int _analysisStart;

	//! Ende der Messungen fuer stochasischen Analyse.
	unsigned int _analysisEnd;

//
// Status-Informationen
//
	
	//! Status der Simulation.
	StateInfo _state;

	//! Aktive Netzkonfiguration.
	Configuration* _activeConfig;
	
	//! Simulationstakt.
	unsigned int _globalTick;

	//! Takte bis zur naechsten Zustandsaenderung.
	unsigned int _stateTicks;
};


//! Zugriff auf Takt der Simulation.
inline unsigned int Tick()
{
	return Simulation::Instance()->Tick();
}

//! Zugriff auf Status der Simulation.
inline Simulation::StateInfo State()
{
	return Simulation::Instance()->State();
}

#endif
