#include <wirematrixmodifier.h>
#include <utility/factory.h>

class Packet;
class Arbiter;
class WireMatrix;
class Wire;
	
//! Modifier zum Entfernen von Reservierungen.
/*!
 * Nachdem der Tail eines Paketes ueber eine bestehende Schaltung vermittelt
 * wurde, kann diese entfernt werden. Diese Klasse stellt dazu einen Modifier
 * fuer die Schaltmatrix eines Routers zu Verfuegung. Es wird sowohl die
 * Matrix aktualisiert, als auch die Schaltung (d.h. angelegtes Register
 * auf ausgehender Leitung) entfernt.
 * 
 * @see WireMatrixModifier
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ClearSwitching : public WireMatrixModifier
{
public:

	//! Implementierung der abstrakten Basisklasse WirematrixModifier.
	/*!
	 * Die Bits O_BUSY und I_BUSY werden in Eintraegen zu einem Ausgang
	 * zurueckgesetzt und das Register von der ausgehenden Leitung entfernt.
	 *
	 * @param it Iterator zu Eintrag in Schaltmatrix eines Routers.
	 */
	void modify(WireMatrixIterator* it);
};

//! Abstrakte Basisklasse fuer Switching-Verfahren der Simulation.
/*!
 * Switching-Verfahren in Kommunikationsnetzen legen fest, wie eine Nachricht
 * den vom Routing vorgegebenen Pfad von Quelle zu Ziel(en) durchlaeuft. Dazu
 * wird festgelegt, wie ein Pfad an einem Schalter allokiert wird und ob und 
 * wie eine Nachricht zerlegt wird. Bei einer Simulation mit CINSim werden stets
 * Pakete betrachtet, die aus Flits bestehen. Die Anzahl der Flits eines Paketes
 * beschreibt die Groesse und pro Takt kann genau ein Flit ueber eine Leitung
 * bewegt werden. Diese Klasse stellt die Strukturen zur Verfuegung, um an
 * einem Router einen Eingang auf einen Ausgang zu schalten und diese Schaltung
 * ggf. ueber mehrere Takte aufrecht zu erhalten. Zur Beschreibung einer
 * Switching-Strategie muessen die speziellen Eigenschaften von Paketen und
 * Flits der jeweilgen Strategie formuliert und eingebunden werden.
 *
 * @see WireMatrixModifier
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Switching : public WireMatrixModifier, public FactoryElement
{
public:

	//! Erzeugen eines passenden Paketes.
	/*!
	 * Diese Methode muss von allen Switching-Verfahren implementiert werden,
	 * so dass die jeweils benoetigten Paketstrukturen generiert werden.
	 *
	 * @param psize Die Groesse des zu generierenden Paketes.
	 * @return Spezielles Paket fuer Switching-Strategie. 
	 */
	virtual Packet* createPacket(unsigned int psize) = 0;

	//! Implementierung der abstrakten Basisklasse WirematrixModifier.
	/*!
	 * Das Schalten eines Ausganges darf nur erfolgen, wenn zuvor ein
	 * Scheduling durchgefuehrt wurde, d.h. Bit O_SCHEDULED ist gesetzt,
	 * und Ausgang fuer aktuellen Takt noch nicht fertig bearbeit ist, 
	 * d.h. Bit O_FINISHED ist nicht gesetzt.
	 *
	 * @param it Iterator zur Eintrag in Schaltmatrix eines Routers.
	 * @return WireMatrixModifier::PASSED, falls neue Schaltung angelegt
	 * werden kann, WireMatrixModifier::FAILED sonst.
	 */
	int precondition(WireMatrixIterator* it) const;
	
	//! Implementierung der abstrakten Basisklasse WirematrixModifier.
	/*!
	 * Das Switching markiert durch Scheduling festgelegtes Paar aus Ein-
	 * und Ausgang als geschaltet, d.h. Bit IO_SWITCHED wird gesetzt, und
	 * legt das Register der einghenden Leitung auf die ausgehende Leitung,
	 * so dass eine Schaltung hergestellt wird. Der Netzkoten am Ende der
	 * ausgehenden Leitung erlangt damit Zugriff auf zu vermittelnde Flits
	 * (Paket) am Startknoten der eingehenden Leitung. Weiterhin werden die
	 * Bits O_FINISHED und O_BUSY fuer alle Eintraege zum geschalteten
	 * Ausgang gesetzt und I_BUSY und I_FINISHED fuer den Eintrag zum
	 * geschalteten Ein-/Ausgangspaar.
	 *  
	 * @param it Iterator zur Eintrag in Schaltmatrix eines Routers.
	 */
	void modify(WireMatrixIterator* it);

	//! Modifier zum Entfernen einer Schaltung.
	/*!
	 * Diese Methode liefert einen Modifier zum Enfernen einer Schaltung,
	 * die mit einer Instanz dieser Klasse erzeugt wurde.
	 *
	 * @return WireMatrixModifier zum Entfernen einer Schaltung.
	 */
	WireMatrixModifier* clear() const;
	
private:

	//! Modifier zum Entfernen von Reservierungen.
	ClearSwitching cs_modifier;
};


