#ifndef __TARGETBUFFER_H__
#define __TARGETBUFFER_H__

#include <buffer.h>

//! Die Ziele eines Verbindungsnetzes.
/*!
 * Diese Klasse beschreibt eine spezielle Variante von Puffern, die als
 * Ziele in einem Kommunikationsnetz dienen. Zielpuffer haben genau einen 
 * Vorgaenger- und keinen Nachfolgerknoten im Netzgraphen. Ihre Bitmaske 
 * maskiert genau ein (eindeutiges) Ziel. Sie besitzen eine maximal
 * grosse Queue, so dass sie aus Sicht sendebereiter, ueber eine Leitung
 * angeschlossenener Vorgaengerknoten stets Flits von Paketen aufnehmen 
 * koennen. Tatsaechlich gelangen die Flits aus Effizienzgruenden aber nicht 
 * in die Queue, sondern werden lediglich fuer Aktualisierung der Messungen 
 * analysiert und im Falle eines Tails wird anschliessend das gesamte Paket 
 * freigegeben. Die Koroutine hat dafuer zu sorgen, dass pro Takt nur ein 
 * Flit empfangen wird.
 * 
 * @see Component
 * @see TriggerNode
 * @see Buffer
 *
 * An Zielpuffern koennen Messungen zum Durchsatz (Flits), sowie zu Latenz-
 * und Verzoegerungszeiten von Paketen in einem Kommunkationsnetz durchgefuehrt
 * werden.
 * 
 * @see Analysable
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class TargetBuffer : public Buffer
{
friend class XmlSimulation;	// Initialisierung
public:
	
	//! Konstruktor.
	/*!
	 * Es erfolgen lediglich notwendige Initialisierungen der Klassenattribute.
	 * In seiner Eigenschaft als XMLElement kann ein zugehoeriges DOMElement
	 * fuer den Zielpuffer spezifiziert werden. Der Abschluss der Initialisierung
	 * eines Zielpuffers erfolgt erst durch Aufruf der Methode init.
	 *
	 * @see XMLElement
	 *
	 * @param scope Zugehoeriges DOMElement mit Beschreibung dieses Zielpuffers.
	 */
	TargetBuffer(DOMElement* scope=NULL) : Buffer(scope) {}

	//! Implementierung der Basisklasse XMLElement.
	virtual bool init(const DOMElement* scope);
	
	//! Implementierung der Basisklasse Analysable.
	/*!
	 * Liefert aktuelle Messwerte fuer Durchsatz, Latenz- und
	 * Verzoegerungszeiten.
	 *
	 * @see Analysable
	 */	
	virtual double Measure( TypeInfo type );
	
	//! Implementierung der Basisklasse Analysable.
	/*!
	 * Liegt fuer ueber erstes Argument sezifizierte Messung ein neuer
	 * Messwert vor, so wird Messung angestossen. Es wird dabei geprueft,
	 * ob Messung zulaessig, neuer Messwert vorhanden und fuer Latenz,
	 * Verzoegung und Durchsatz ggf. die geforerte Prioritaet uebereinstimmt.
	 *
	 * @see Analysable
	 */
	virtual void takeMeasure(const Description& descr);
        
	/*! 
	 * Implementierung der Basisklasse Component.
	 *
	 * @see Component
	 * @return Component::TARGET
	 */
	virtual const int rtti() const
	{
		return Component::TARGET;
	}

	/*! 
	 * Implementierung der Basisklasse Component.
	 *
	 * @see Component
	 * @return "Target"
	 */
	virtual const std::string Name() const
	{
		return std::string ("Target");
	}

	//! Implementierung der Basisklasse Component.
	/*!
	 * In der Bitmaske eines Zielpuffers wird nur das Bit gesetzt, dass
	 * dem Index des Zieles entspricht.
	 *
	 * @see TargetBuffer::targetIndex
	 * @see Component
	 */
	virtual const Bitmask& generateBitmask();

	//! Implementierung der Basisklasse Component.
	/*!
	 * Fuer ein Ziel ist hier nichts zu tun. Pakete werden sofort
	 * geloescht, wenn das letzte Flit empfangen wurde und alle
	 * Messungen aktualisert wurden.
	 * 
	 * @see Component
	 */
	virtual void reset();

	//! Implementierung der Basisklasse Buffer.
	/*!
	 * Wird ein Flit empfangen, so werden die Messwerte fuer Latenz- und 
	 * Verzoegerungszeiten, sowie fuer den Durchsatz aktualisiert. Wird
	 * das Ende eines Paketes empfangen, so wird dieses anschliessend 
	 * freigegeben. Pakete bzw. Flits werden an Zielen nicht gepuffert.
	 * 
	 * @see Buffer
	 */
	virtual void flitReceived(Flit *flit);
		
private:

	//! Durchsatz an einem Zielpuffer.
	/*!
	 * Durchsatz an Flits an einem Zielpuffer. Der Wert wird auf 1 gesetzt,
	 * wenn ein Flit empfangen wird. Ansonsten wird er als 0 interpretiert.
	 * 
	 * @see TargetBuffer::flitReceived
	 */
	Data _throughput;
};

#endif
