#ifndef __BITMASK__
#define __BITMASK__

#include <vector>
#include <iostream>
#include <bits/wordsize.h>

//FIXME: statt modulo sollte ein bitshift von 5 nach rechts schneller sein?
#define FIELD_INDEX(bit) ((unsigned int)((bit) / (__WORDSIZE)))
#define BIT_INDEX(bit) ((unsigned int)((bit) % (__WORDSIZE)))


//! Bitmasken beliebiger Groesse.
/*!
 * Diese Klasse realisiert Bitmasken fuer eine beliebige Anzahl von
 * Bits basierend auf einem Array von vorzeichenlosn Integern (unsigned int),
 * die den Speicherbereich der Bits bilden.
 * 
 * @author Matthias Kuehm<kuehmi@cs.tu-berlin.de>
 */
class Bitmask
{
public:

	//! Konstruktor.
	/*!
	 * Der Konstruktor einer Bitmaske bekommet als (optionales) Argument
	 * die erwartete Anzahl von zu verwaltenden Bits. Es wird in jedem
	 * Fall Speicher fuer Bits in Groesse eines vorzeichenlosen Integers 
	 * (unsigned int) bereitgestellt.
	 *
	 * @param expected Erwartete Anzahl von zu verwaltenden Bits.
	 */
	Bitmask(unsigned int expected=0) : farray(FIELD_INDEX(expected?expected-1:expected)+1) {}

	//! Copy-Konstruktor.
	/*!
	 * Der Copy-Konstruktur erstellt eine tiefe Kopie der des internen
	 * Arrays der uebergebenen Bitmaske.
	 *
	 * @param bitmask Vorlage fuer neue Bitmaske.
	 */
	Bitmask(const Bitmask& bitmask) : farray(bitmask.farray.size)
	{
		farray=bitmask.farray;	
	}
	
	//! Anzahl der alloziierten Bitfelder.
	/*!
	 * Der Speicherbereich fuer die zu verwaltenden Bits wird von einem
	 * Array von unsigned int (@see FieldArray) zur Verfuegung gestellt.
	 * Diese Methode liefert die Anzahl der allozierten Felder.
	 *
	 * @return Anzahl von allozierten Feldern der Groesse sizeof(unsigned int).
	 */
	unsigned int fields() const
	{
		return farray.size;
	}
	
	//! Setzen eines Bits.
	/*!
	 * Diese Methode setzt das uebergebene Bit und alloziert bei Bedarf
	 * neuen Speicher.
	 *
	 * @param bit Das zu setzende Bit.
	 */
	void set(unsigned int bit);

	//! Loeschen eines Bits.
	/*!
	 * Diese Methode entfernt ein gesetztes Bit.
	 *
	 * @param bit Das zu entfernende Bit.
	 */
	void reset(unsigned int bit);

	//! Test auf gesetztes Bit.
	/*!
	 * Diese Methode prueft, ob das uebergebene Bit gesetzt ist. Liegt das
	 * Bit ausserhalb des aktuell verwalteten Bereichs, wird false geliefert.
	 *
	 * @param bit Das zu testende Bit.
	 * @return true, falls zu testendes Bit gesetzt.
	 */
	bool test(unsigned int bit) const;
	
	//! Test auf leere Bitmaske.
	/*!
	 * Diese Methode prueft, ob keines der verwalteten Bits gesetzt ist.
	 *
	 * @return true
	 */
	bool empty() const;

	//! Zuruecksetzen aller Bits.
	/*!
	 * Diese Methode setzt alle verwalteten Bits der Bitmaske zurueck.
	 */
	void clear();

	//! Test auf nichtleeren Schnitt.
	/*!
	 * Dieser kommutative Operator prueft, ob zwei Bitmasken mindestens ein 
	 * gemeinsam gesetztes Bit besitzen.  
	 *
	 * @return true, falls Bit existiert, das in beiden Bitmasken gesetzt ist.
	 */
	bool operator<=(const Bitmask& op) const;
	
	//! Test auf identische Bitmasken.
	/*!
	 * Dieser kommutative Operator prueft, ob zwei Bitmasken identisch sind,
	 * d.h. in allen gesetzten Bits uebereinstimmen.
	 *
	 * @return true, falls Bitmasken in gesetzten Bits uebereinstimmen.
	 */
	bool operator==(const Bitmask& op) const;

	//! Bitweiser Schnitt zweier Bitmasken.
	/*!
	 * Dieser kommutative Operator liefert den Schnitt der gesetzten Bits
	 * zweier Bitmasken beliebiger Groesse in Form einer neuer Bitmaske. Die 
	 * Groesse der neuen Bitmaske orientiert sich am Maximum der gegebenen 
	 * Bitmasken.
	 * 
	 * @return Ergebnis des bitweisen Schnitts zweier Bitmasken.
	 */
	Bitmask operator&(const Bitmask& op) const;
	
	//! Bitweise Vereinigung zweier Bitmasken.
	/*!
	 * Dieser kommutative Operator liefert die Vereinigung der gesetzten Bits
	 * zweier Bitmasken beliebiger Groesse in Form einer neuer Bitmaske. Die 
	 * Groesse der neuen Bitmaske orientiert sich am Maximum der gegebenen 
	 * Bitmasken.
	 * 
	 * @return Ergebnis der bitweisen Vereinigung zweier Bitmasken.
	 */
	Bitmask operator|(const Bitmask& op) const;
	
	// Bitweises exklusive Vereinigung zweier Bitmasken.
	/*!
	 * Dieser kommutative Operator liefert die exklusive Vereinigung der 
	 * gesetzten Bits zweier Bitmasken beliebiger Groesse in Form einer neuer 
	 * Bitmaske. Die Groesse der neuen Bitmaske orientiert sich am Maximum der 
	 * gegebenen Bitmasken.
	 * 
	 * @return Ergebnis der bitweisen exklusiven Vereinigung zweier Bitmasken.
	 */
	Bitmask operator^(const Bitmask& op) const;

	//! Bitweise Differenz zweier Bitmasken.	
	/*!
	 * Dieser nicht-kommutative Operator liefert die Differenz der gesetzten 
	 * Bits zweier Bitmasken beliebiger Groesse in Form einer neuer Bitmaske.
	 * Die Groesse der neuen Bitmaske orientiert sich am Maximum der gegebenen 
	 * Bitmasken.
	 * 
	 * @return Ergebnis der bitweisen Differenz zweier Bitmasken.
	 */
	Bitmask operator-(const Bitmask& op) const;

	//! Bitweiser Schnitt zweier Bitmasken mit Zuweisung.
	/*!
	 * Dieser Operator berechnet den Schnitt der gesetzten Bits zweier Bitmasken 
	 * beliebiger Groesse und modifiziert den ersten Operanden. Die Groesse des 
	 * ersten Operanden wird bei Bedarf erhoeht.
	 * 
	 * @return Ergebnis des bitweisen Schnitts zweier Bitmasken.
	 */
	Bitmask& operator&=(const Bitmask& op);

	//! Bitweise Vereinigung zweier Bitmasken mit Zuweisung.
	/*!
	 * Dieser Operator berechnet die Vereinigung der gesetzten Bits zweier Bitmasken 
	 * beliebiger Groesse und modifiziert den ersten Operanden. Die Groesse des 
	 * ersten Operanden wird bei Bedarf erhoeht.
	 * 
	 * @return Ergebnis der bitweisen Vereinigung zweier Bitmasken.
	 */
	Bitmask& operator|=(const Bitmask& op);
	
	//! Bitweise exklusive Vereinigung zweier Bitmasken mit Zuweisung.
	/*!
	 * Dieser Operator berechnet die exklusive Vereinigung der gesetzten Bits 
	 * zweier Bitmasken beliebiger Groesse und modifiziert den ersten Operanden. 
	 * Die Groesse des ersten Operanden wird bei Bedarf erhoeht.
	 * 
	 * @return Ergebnis der bitweisen exklusiven Vereinigung zweier Bitmasken.
	 */
	Bitmask& operator^=(const Bitmask& op);
	
	//! Bitweise Differenz zweier Bitmasken mit Zuweisung.
	/*!
	 * Dieser Operator berechnet die Differenz der gesetzten Bits zweier Bitmasken 
	 * beliebiger Groesse und modifiziert den ersten Operanden. Die Groesse des 
	 * ersten Operanden wird bei Bedarf erhoeht.
	 * 
	 * @return Ergebnis der bitweisen Differenz zweier Bitmasken.
	 */
	Bitmask& operator-=(const Bitmask& op);

private:
	
	//! Dynamisches Array fuer Bitfelder.
	/*!
	 * Diese Klasse stellt ein einfaches dynamisches Array zur Verfuegung, das
	 * einen Speicherbereich fuer Bitfelder zur Verfuegung stellt. Die Groesse
	 * eines Bitfeldes, d.h. die Anzahl der Bits, orientiert sich an der Groesse
	 * des Datentyps unsigned int.
	 * 
	 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
	 */
	struct FieldArray
	{
		//! Konstruktor.
		/*!
		 * Es wird mindestens Speicher fuer ein Bitfeld alloziiert,
		 * optional kann ein groesserer Wert uebergeben werden.
		 * 
		 * param fields Die Anzahl der verwalteten Bitfelder.
		 */
		FieldArray(unsigned int fields=1);
	
		//! Destruktor.
		~FieldArray() { delete[] inner_array; }
		
		//! Dimensionsaenderung.
		/*!
		 * Das interne Array wird an die neue Anzahl von zu verwaltenden
		 * Bitfelder angepasst, wobei neuer Speicher alloziiert oder Speicher
		 * freigegeben wird. Die Mindestgroesse von einem Bitfeld bleibt
		 * erhalten.
		 *
		 * @param fields Neue Zahl zu verwaltender Bitfelder.
		 */
		void resize(unsigned int fields);

		//! Lesender, Selektiver Zugriff auf ein Bitfeld.
		/*!
		 * Diese Methode liefert eine Kopie des Bitfelds zu uebergebenem
		 * Index oder 0 bei Bereichsueberschreitung.
		 * 
		 * @param field Index des zu lesenden Bitfeldes.
		 * @return Bitfeld (Kopie) zu spezifiziertem Index oder 0.
		 */
		unsigned int read(unsigned int field) const
		{
			if(field>=size)
				return 0;

			return *(inner_array+field);
		}
	
		//! Selektiver Zugriff auf ein Bitfeld mit Speicheralloziierung.
		/*!
		 * Diese Methode liefert eine Referenz des Bitfelds zu uebergebenem
		 * Index und alloziiert bei Bereichsueberschreitung neuen Speicher.
		 * 
		 * @param field Index des zu lesenden Bitfeldes.
		 * @return Bitfeld zu spezifiziertem Index.
		 */
		unsigned int& operator[](unsigned int field)
		{
			if(field>=size) resize(field+1);

			return *(inner_array+field);
		}
		
		//! Selektiver Zugriff auf ein Bitfeld.
		/*!
		 * Diese Methode liefert eine Referenz des Bitfelds zu uebergebenem
		 * Index ohne auf Bereichsueberschreitung zu testen.
		 * 
		 * @param field Index des zu lesenden Bitfeldes.
		 * @return Bitfeld zu spezifiziertem Index.
		 */
		unsigned int& operator()(unsigned int field)
		{
			return *(inner_array+field);
		}

		//! Zuweisungs-Operator.
		/*!
		 * Erstellt tiefe Kopie des zweiten Parameters und weist sie
		 * dem ersten zu.
		 *
		 * @param op Zuzueisendes Array.
		 */
		FieldArray& operator=(const FieldArray& op);

		//! Internes Array fuer die Bitfelder.
		unsigned int* inner_array;

		//! Groesse des internen Arrays.
		unsigned int size;
		
	} farray;
};

inline std::ostream& operator<<(std::ostream& target, const Bitmask& toShow)
{
	unsigned int i=0;
	unsigned int j=0;
	
	for(; i<toShow.fields(); i++)
	{
		target << "{" << toShow.test(j++);
		
		for(; (j % __WORDSIZE); j++)
		{
			if(!(j%8))
				target << " ";
			
			target << toShow.test(j);
		}

		target << "}";	
	}
	return target;
}

#endif
