#ifndef __VALUE_STRING__
#define __VALUE_STRING__

#include <string>
#include <stdlib.h>

//! Automatisierte Stringkonvertierung.
/*!
 * Diese Klasse realisiert Konvertierungen eines gegebenen Strings in 
 * verschiedene integrale Datentypen durch Bereitstellung verschiedener
 * Cast-Operatoren.
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ValueString
{
public:
	
	//! Konstruktor.
	/*!
	 * Der Konstruktoraufruf initialisiert eine Instanz dieser Klasse anhand 
	 * eines gegebenen Strings.
	 *
	 * @param value Ein String.
	 */
	ValueString(const std::string& value);

	//! Argumentloser Konstruktor.
	/*!
	 * Der Aufruf dieses Konstruktors ist gleichbedeutend mit dem Aufruf des
	 * String-Konstruktors mit einem leeren String als Argument. Die Casts
	 * nach int, unsigned int und double liefern 0.
	 */
	ValueString() : m_str(""), real_string(false), negative(false) {}
	
	//! Destruktor.
    ~ValueString() {}

	//! Konvertierung nach double.
	/*!
	 * Cast-Operator zur Konvertierung nach double. Abbruch mit exit, 
	 * falls Cast nicht moeglich.
	 */
	operator double() const;

	//! Konvertierung nach int.
	/*!
	 * Cast-Operator zur Konvertierung nach int. Abbruch mit exit, 
	 * falls Cast nicht moeglich.
	 */
	operator int() const;

	//! Konvertierung nach unsigned int.
	/*!
	 * Cast-Operator zur Konvertierung nach unsigned int. Abbruch mit exit, 
	 * falls Cast nicht moeglich.
	 */
	operator uint() const;
   
	//! Konvertierung nach string.
	/*!
	 * Dieser Cast-Operator liefert den eingebetteten String.
	 */
    operator std::string() const
    {
		return m_str;
    }

	/*!
	 * Test auf numerischen Wert.
	 * 
	 * return true, falls String eine Zahl kodiert.
	 */
	inline bool isNumber() const
	{
		return !real_string;
	}
	
	/*!
	 * Test auf positiven numerischen Wert.
	 * 
	 * return true, falls String eine positive Zahl kodiert.
	 */
	inline bool isPositiveNumber() const
	{
		return !real_string && !negative;
	}

protected:

	//! Initialisierung.
	/*!
	 * Es wird eine Kopie des uebergebenen Strings angelegt und untersucht,
	 * ob es sich um einen numerischen String handelt.
	 *
	 * @param value Ein String.
	 */
	void setString(const std::string& value);
	
private:

	//! Eingebetteter String.
	std::string m_str;	
	//! Flag fuer nicht-numerischen String.
	bool real_string;
	//! Flag fuer negatives Vorzeichen.
	bool negative;
};

inline std::ostream & operator<<(std::ostream & out, ValueString& att)
{
     out << (std::string)att;
     return out;
};


#endif
