#ifndef __XML_ELEMENT__
#define __XML_ELEMENT__

#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOM.hpp>

XERCES_CPP_NAMESPACE_USE;

class XMLAttribute;

//! Einbettung von XML-Elementen in CINSim-Komponenten.
/*!
 * Alle Komponenten von CINSim, die ueber die Attribute eines XML-Elementes und
 * dessen Subelemente aus der Simulationsbeschreibung initialisiert werden sollen, 
 * implementieren diese abstrakte Basisklasse. Das entsprechende Element wird 
 * eingebettet um auf vorhandene Attribute und Subelemente aus diesem 
 * Gueltigkeitsbereich jederzeit zugreifen zu koennen.
 *
 * @see XMLAttribute
 *
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class XMLElement
{
public:

	//! Standard-Konstruktor.
	/*!
	 * Der Aufruf des Konstruktors initilisiert den Gueltigkeitsbereich
	 * innerhalb des zugehoerigen XML-Dokumentes.
	 * 
	 * @param scope Der Gueltigkeitsbereich.
	 */
	XMLElement(const DOMElement* scope=NULL);

	//! Destruktor.
	virtual ~XMLElement();

	//! Initialisierung.
	/*!
	 * Diese Methode wird von allen abgeleiteten Klassen implementiert
	 * und dient der Initialisierung anhand des Gueltigkeitsbereiches im
	 * XML-Dokument.
	 * 
	 * @param scope Der Gueltigkeitsbereich.
	 * @return true, falls Initialisierung erfolgreich.
	 */
	virtual bool init(const DOMElement* scope=NULL) = 0;

	//! Zugriff auf Attribut in Gueltigkeitsbereich.
	/*!
	 * Diese Methode bietet Zugriff auf Werte von Attributen innerhalb
	 * des Gueltikeitsbereiches eines XMLElements. Optional kann ein
	 * ein abweichender Gultikeitsbereich spezifiziert werden, um 
	 * beispielsweise Attribute von Subelementen zu erreichen. Der aktuelle
	 * Gueltigkeitsbereich bleibt unveraendert.
	 *
	 * @see XMLAttribute
	 * 
	 * @param name Name des Attributes.
	 * @param scope (Optionaler) Gueltigkeitsbereich.
	 * @return Wert eines Attributes aus Gueltigkeitsbereich.
	 */
	XMLAttribute getAttribute(const char* name, const DOMElement* scope=NULL) const;
	
	//! Zugriff auf Parent in Gueltigkeitsbereich.
	/*!
	 * Diese Methode bietet Zugriff auf den uebergeordneten Knoten im
	 * DOM-Baum fuer dieses XML-Element. Optional kann auch ein
	 * ein abweichender Gultikeitsbereich spezifiziert werden. Der 
	 * aktuelle Gueltigkeitsbereich bleibt unveraendert.
	 *
	 * @see XMLAttribute
	 * 
	 * @param scope (Optionaler) Gueltigkeitsbereich.
	 * @return Parent-Node oder NULL, falls kein Parent vorhanden.
	 */
	const DOMElement* getParent(const DOMElement* scope=NULL) const;
	
	//! Gueltigkeitsbereich setzen.
	/*!
	 * Diese Methode setzt den Gueltigkeitsbereich innerhalb eines XML-Dokumentes.
	 * Der Gueltigkeitsbereich beginnt bei einem Element des DOM-Baumes und 
	 * bezieht dessen Attribute und eventuell vorhandene Subelemente ein.
	 *
	 * @param scope Der (neue) Gueltigkeitsbereich.
	 */
	void setScope(const DOMElement* scope)
	{
		this->scope = scope;
	}
	
	//! Gueltigkeitsbereich.
	/*!
	 * Zugriff auf Gueltigkeitsbereich eines XMLElements.
	 * 
	 * @return Pointer auf DOMElement oder NULL, falls Gueltikeitsbereich undefiniert.
	 */
	const DOMElement* Scope() const { return scope; };

private:

	//! Element des DOM-Baumes welches den Gueltikeitsbereich definiert.
	const DOMElement* scope;
};


#endif
