#ifndef __XMLHELPER_H__
#define __XMLHELPER_H__

#include <utility/xmlattribute.h>
#include <environment/parser/cinparser.h>

#include <vector>
#include <iostream>

using std::vector;

bool equals(const XMLCh*, const char*);
bool endsWith(const XMLCh*, const char*);

//! Filter fuer XML-Elemente mit gegebenen Namen.
/*!
 * Diese Klasse implementiert einen Filter fuer XML-Elemente zur Verwendung
 * fuer die Dokument-Traversierung (DOMTreeWalker oder DOMNodeIterator). Es
 * werden nur Elemente akzepiert deren Name dem spezifizierten Namen entspricht.
 */

class ElementFilter : public DOMNodeFilter
{
public:
	
	ElementFilter(const char* tagname=NULL, bool (*filter)(const XMLCh*, const char*) = equals)
		: tagname(tagname), filter(filter) {};
			
	// works with xerces version 2.7.x and lower 
	#if _XERCES_VERSION >= 30000
	  virtual DOMNodeFilter::FilterAction acceptNode(const DOMNode* node) const;
	// xerces version 3.x.x 
	#else
	  virtual short acceptNode(const DOMNode* node) const;
	#endif
	
	
		
protected:
		
	const char* tagname;

	bool (*filter)(const XMLCh*, const char*);		
};

//! Filter fuer XML-Elemente mit globalen/lokalen Einstellungen.
/*!
 * Diese Klasse implementiert einen Filter fuer XML-Elemente zur Verwendung
 * fuer die Dokument-Traversierung (DOMTreeWalker oder DOMNodeIterator). Es
 * werden nur Elemente akzepiert deren Namen mit 'settings' enden, also die
 * XML-Elemente, die Standard-Einstellungen fuer bestimmte Komponenten global
 * oder lokal festlegen.
 */
class SettingsFilter : public ElementFilter
{
public:
	SettingsFilter() : ElementFilter("settings", endsWith) {}
};


//! Wrapper-Klasse fuer Stream-Ausgabe von XMLCh*.
/*!
 * Variablen/Werte vom Typ XMLCh* koennen durch Aufruf StrX(var) in
 * einen Stream fuer formatierte Ausgaben geschrieben werden.
 */
class StrX
{
public:
	
	//!  Konstruktor.
	StrX(const XMLCh* const toTranscode)
	{
		fLocalForm = XMLString::transcode(toTranscode);
	}

	//! Destruktor.
	~StrX()
	{
		XMLString::release(&fLocalForm);
	}

	//! Interner C-String.
	/*!
	 *
	 * return Interner C-String zu Xerces-String.
	 */
	const char* localForm() const
	{
		return fLocalForm;
	}

private:

	//! Interner C-String.
	char*   fLocalForm;
};


/*!
 * (Hilfs)Methode zum Stringvergleich 
 */
bool equals(const XMLCh* str1, const char* str2);


/*!
 * (Hilfs)Methode zum Stringvergleich
 */
bool equalsIgnoreCase(const XMLCh* str1, const char* str2);

	
/*!
 * (Hilfs)Methode zum Stringvergleich
 */
bool endsWith(const XMLCh* str1, const char* str2);


/*!
 * (Hilfs)Methode zum Stringvergleich
 */
bool startsWith(const XMLCh* str1, const char* str2);


/*!
 * Konvertierungsfunktion C-String -> Xerces-String
 */
const XMLCh* conv(const char* str);


/*!
 * Konvertierungsfunktion Xerces-String -> C-String 
 */
char* conv(const XMLCh* str);


//! Parsierungsmethode fuer Attribute.
/*!
 * Diese Methode parsiert ein Attribut eines XML-Elements und erstellt Instanz
 * der Klasse XMLAttribute. Wird Attribut nicht gefunden so wird abgebrochen.
 *
 * @see XMLAttribute
 *
 * @param elem Der betrachtete Elemenent-Knoten.
 * @param name Der Name des Attributs.
 * @return Attributwert (Integer, Double, ...).
 */
XMLAttribute parseAttribute(const DOMElement* elem, const char* name);


//! Vektor mit direkten Subelementen spezifizierten Namens.
/*!
 * Diese Methode bestimmt mit Hilfe eines TreeWalkers die direkten Subelemente
 * gegebenen Namens eines XML-Elements. Der Name kann durch Spezifikation einer
 * entsprechenden Methode auch als Prefix oder Suffix interpretiert werden.
 *
 * @param parent Das Element, dessen Subelemente untersucht werden sollen.
 * @param tagname Der Name der gesuchten Subelemente.
 * @param deep Flag fuer Verzweigung in Unterbaeume.
 * @param filter Filterfunktion.
 * @return Liste von Pointern auf gesuchte Subelemente.
 */
vector<DOMElement*> getChildElementsByName(DOMElement* parent, const char* tagname,
		bool deep = false, bool (*filter)(const XMLCh*,const char*) = equals);


//! Erstes Subelement mit spezifiziertem Namen.
/*!
 * Diese Methode bestimmt mit Hilfe eines TreeWalkers das erste direkte Subelement
 * gegebenen Namens eines XML-Elements. Der Name kann durch Spezifikation einer
 * entsprechenden Methode auch als Prefix oder Suffix interpretiert werden.
 *
 * @param parent Das Element, dessen Subelemente untersucht werden sollen.
 * @param tagname Der Name der gesuchten Subelemente.
 * @param filter Filterfunktion.
 * @return Pointer auf gesuchtes Subelement.
 */
DOMElement* getFirstChildByName(const DOMElement* parent, const char* tagname, 
		bool (*filter)(const XMLCh*,const char*) = equals);


//! Nachbarknoten mit spezifiziertem Namen.
/*!
 * Diese Methode bestimmt mit Hilfe eines TreeWalkers den ersten Nachbarknoten
 * gegebenen Namens. Wird keiner gefunden so wird element zurueckgeliefert. Der
 * Name kann durch Spezifikation einer entsprechenden Methode auch als Prefix 
 * oder Suffix interpretiert werden.
 *
 * @param element Das Element, dessen Nachbarn untersucht werden sollen.
 * @param tagname Der Name der gesuchten Subelemente.
 * @param filter Filterfunktion.
 * @return Pointer auf gesuchtes Subelement oder Ausgangselement.
 */
DOMElement* getNextSiblingByName(DOMElement* element, const char* tagname,
		bool (*filter)(const XMLCh*,const char*) = equals);


//! TreeWalker fuer Elementknoten.
/*!
 * Diese Methode liefert einen TreeWalker, der die Subelemente von Knoten root
 * traversiert. Wird deep=true spezifiziert, so wird der gesamte Baum unterhalb
 * von root betrachtet, andernfalls nur die direkten Nachfolger. Argument tagname 
 * spezifiziert gesuchte Element, tagname=NULL lieferte alle Elemente.
 *
 * FIXME: deep scheint keine Wirkung zu haben 
 *
 * @param root Ausgangselement fuer Baumtraversierung.
 * @param tagname Gesuchte Elemente, NULL liefert alle.
 * @param deep Flag fuer Verzweigung in Unterbaeume.
 * @param  filter Filterfunktion.
 * @return Pointer auf TreeWalker fuer Elemente.
 */
DOMTreeWalker* createElementWalker(DOMElement* root, const char* tagname, 
		bool deep, bool (*filter)(const XMLCh*,const char*) = equals);

//! Iterator fuer Elementknoten.
/*!
 * Diese Methode liefert einen Iterator, der die Subelemente von Knoten root
 * traversiert. Wird deep=true spezifiziert, so wird der gesamte Baum unterhalb
 * von root betrachtet, andernfalls nur die direkten Nachfolger. Argument 
 * tagname spezifiziert gesuchte Element, tagname = NULL lieferte alle Elemente.
 * Im Unterschied zum TreeWalker werden auch Subelemente von Elementen, die den
 * Filter passieren, betrachtet.
 *
 * FIXME: deep scheint keine Wirkung zu haben 
 * 
 * @param root Ausgangselement fuer Baumtraversierung.
 * @param tagname Gesuchte Elemente, NULL liefert alle.
 * @param deep Flag fuer Verzweigung in Unterbaeume.
 * @param filter Filterfunktion.
 * @return Pointer auf TreeWalker fuer Elemente.
 */
DOMNodeIterator* createNodeIterator(DOMElement* root, char* tagname, 
		bool deep, bool (*filter)(const XMLCh*,const char*) = equals);

//! Attribut-Merge fuer Element-Knoten.
/*!
 * Diese Methode fuegt dem zweiten Argument alle Attribut-Knoten hinzu,
 * die im ersten Argument vorhanden sind, aber im zweiten nicht. Ist
 * eines der Argumente NULL, so wird dies als Fehler interpretiert.
 *
 * @param elem1 XML-Element mit 'vollstaendigen' Attributen.
 * @param elem2 XML-Element mit 'fehlenden' Attributen.
 * @return 0 - Fehler, 1 - Erfolg.
 */
int mergeElements(const DOMElement* elem1, DOMElement* elem2);

//! Hinzufuegen eines Attributes.
/*!
 * Diese Methode fuegt einem XML-Element ein Attribut hinzu, sofern
 * dieses noch nicht existiert.
 *
 * @param elem Ein XML-Element.
 * @param attr Neues Attribut.
 * @return 0 - Fehler, 1 - Attribut existiert, 2 - Attribut hinzugefuegt.
 */
int addAttribute(DOMElement* elem, DOMAttr* attr);

//! Typueberpruefung fuer DOM-Element.
/*!
 * Der Einsatz eines XML Schemas zur Beschreibung eines XML-Dokumentes 
 * erlaubt eine Typisierung von DOM-Elementen bis hin zu Ableitungen.
 * Diese Methode prueft ob ein Element Instanz der uebergebenen Klasse
 * von DOM-Elementen ist.
 * 
 * @param sinfo Schema-Informationen.
 * @param element Zu pruefendes DOMElement.
 * @param type Name des zu geforderten (Basis-)Typs.
 * @return true, falls element vom Typ type.
 */
bool isDerivedFrom(const SchemaInformation* sinfo, const DOMElement* element, const char* type);
 
/*!
 * Test auf Netzknoten.
 *
 * @return true, falls element Netzknoten beschreibt.
 */
inline bool isNode(const DOMElement* element)
{
	return element->hasAttribute(conv("type")) 
		&& (std::string)parseAttribute(element, "type") == "node";
}

/*!
 * Test auf Netzkante.
 *
 * @return true, falls element Netzkante beschreibt.
 */
inline bool isConnector(const DOMElement* element)
{
	return element->hasAttribute(conv("type")) 
		&& (std::string)parseAttribute(element, "type") == "connector";
}

/*!
 * Test auf Textelement.
 *
 * @return true, falls element ein Textelement beschreibt.
 */
inline bool isText(const DOMElement* element)
{
	return element->hasAttribute(conv("type")) 
		&& (std::string)parseAttribute(element, "type") == "text";
}

/*!
 * Test auf Netzbeschreibung.
 *
 * @return true, falls element Netz mit Knoten und Kanten beschreibt.
 */
inline bool isNetGraph(const DOMElement* element)
{
	return element->hasAttribute(conv("netclass"));
}

//! Ausgabe eines XML-Strings in Stream.
inline std::ostream& operator<<(std::ostream& target, const StrX& toDump)
{
  target << toDump.localForm();
  return target;
}

#endif
