#ifndef __WIRE_H__
#define __WIRE_H__

#include <component.h>
#include <assert.h>
#include <packet.h>

class Node;
class Wire;
class Flit;
class RegisterContainer;

typedef std::vector<Wire*> WireVector;
typedef std::list<Wire*> WireList;

//! Unidirektionale Leitungen eines Verbindungsnetzes.
/*!
 * Diese Klasse beschreibt die unidirektionalen Leitungen eines
 * Verbindungsnetzes. Ein Leitung ist wie die Netzknoten eine Komponente 
 * eines Verbindungsnetzes und besitzt genau einen Vorgaengerknoten und 
 * genau einen Nachfolgerknoten.
 *
 * @see Component
 * @see Node
 * 
 * Leitungen dienen der Uebertragung von Nachrichten und besitzen aus 
 * diesem Grund einen Pointer auf das Senderegister des letzten Puffers
 * aus Richtung des Vorgaengerknotens, der den Zugriff auf ein 
 * anliegendes Nachrichtenfragment (Flit) aus Richtung des des
 * Nachfolgerknotens erlaubt. Verbindet eine Leitung einen Puffer und
 * einen Router, so ist es Aufgabe des Routers anhand der Ziele einer
 * Nachricht zwei Puffer in Senderichtung zu verbinden, d.h. den Pointer 
 * auf das Senderegister in Senderichtung weiterzugeben.
 * 
 * @see RegisterContainer
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Wire : public Component 
{

friend class XmlSimulation;
friend class Test_RegisterContainer;

public:

	//! Konstruktor.
	/*!
	 * Der Konstruktor erstellt eine Instanz der Klasse Wire, fuer die 
	 * anschliessend noch ein Vorgaenger- und ein Nachfolgerknoten
	 * spezifiziert werden muss. Ein Router besitzt in der Regel 
	 * mehrere ein- und ausgehende Leitungen, denen jeweils separat
	 * fuer Eingaenge und Ausgaenge Indizes zugewiesen. Bei Aufruf
	 * des Konstruktors koennen diese Indizes gesetzt werden oder
	 * aber auch nachtraeglich. Fuer Puffer sind die Indizes auf 0
	 * zu setzen.
	 *
	 * @see setFrom
	 * @see setTo
	 * @see setInIndex
	 * @see setOutIndex
	 * 
	 * @param iindex Position der Leitung in Liste von Eingaengen des
	 * 	angeschlossenen Netzknotens.
	 * @param oindex Position der Leitung in Liste von Ausgaengen des
	 * 	angeschlossenen Netzknotens.
	 */
	Wire(unsigned int iindex=0, unsigned int oindex=0);

	//! Implementierung der abstrakten Basisklasse XMLElement.
	/*!
	 * Fuer eine Wire sind keine weiteren Initialisierungen notwendig.
	 * 
	 * @see XMLElement
	 */
	bool init(const DOMElement* scope=NULL) { return false; };
	
	//! Implementierung der abstrakten Basisklasse Component.
	/*!
	 * Diese Methode liefert den Typ-Identifier der Klasse Wire.
	 * 
	 * return Component::WIRE.
	 */
	const int rtti() const 
	{
		return Component::WIRE; 
	}

	/*!
	 * Implementierung der abstrakten Basisklasse Component.
	 * 
	 * @see Component
	 * @return "Wire".
	 */
	const std::string Name() const
	{
		return  std::string("Wire");
	}
	
	//! Implementierung der abstrakten Basisklasse Component
	/*! 
	 * Eine Wire uebernimmt die Bitmaske ihres Nachfolgerknotens.
	 * 
	 * @see Component
	 */
	const Bitmask& generateBitmask();
	
	//! Implementierung der abstrakten Basisklasse Component.
	/*!
	 * Die Verbindung zum Sendregister eines vorhergehenden Puffers
	 * wird aufgehoben,
	 * 
	 * @see Component
	 */
	virtual void reset() 
	{ 
		unconnectRegister(); 
	}
	
	//! Verbundenes Register.
	/*! 
	 * Das Register auf einer Leitung zeigt an, ob ein Paket ueber 
	 * diese Leitung vermittelt werden soll. Dabei handelt es sich um
	 * einen Pointer auf das Register einer BufferQueue. 
	 * 
	 * return Register dieser Leitung.
	 */
	inline RegisterContainer* RegisterOnLine() const 
	{
		return _reg; 
	}
	
	//! Verbindung zu einem Register herstellen.
	/*!
	 * Diese Methode verbindet diese Leitung mit dem Register einer
	 * BufferQueue. Liegt dort ein Paket an, so liegt es auch auf der 
	 * Leitung.
	 *
	 * @param container Register einer BufferQueue. 
	 */
	void connectRegister(RegisterContainer* container);
	
	//! Verbindung zu einem Register aufheben.
	/*!
	 * Diese Methode sorgt dafuer das die Verbindung zu einem
	 * Register einer BufferQueue geloest wird. Ein eventuell vohandenes
	 * Flit in diesem Register bleibt erhalten.
	 */
	inline void unconnectRegister() 
	{ 
		_reg = NULL; 
	}

	//! Gueltiges Flit auf der Line.
	/*!
	 * Diese Methode liefert einen Pointer auf das anliegende Flit
	 * dieser Leitung, falls existent. Liegt kein Flit an, oder das
	 * anliegende Flit wurde als ungueltig markiert, so wird NULL
	 * zurueckgeliefert.
	 *
	 * @return Pointer auf anliegendes, gueltiges Flit oder NULL.
	 */
	const Flit* getFlit(Node* customer=NULL) const;
	
	//! Anliegendes Nachrichtenpaket.
	/*!
	 * Diese Methode liefert einen Pointer auf das aktuell zu 
	 * vermittelnde Nachrichtenpaket des angeschlossenen Sendregisters
	 * eines Puffers. Besteht keine Verbindung zu einem Senderegister
	 * oder ist kein Paket zu vermitteln, so wird NULL zurueckgeliefert.
	 *
	 * @return Pointer auf anliegendes Nachrichtpaket oder NULL.
	 */
	const Packet* getPacket() const;
	
	//! Eingangsindex der Leitung.
	/*!
	 * Der Index der Leitung aus Sicht des Nachfolgeknotens.
	 * 
	 * @see setInIndex
	 * @return Eingangsindex der Leitung aus Sicht des Nachfolgerknotens.
	 */
	inline unsigned int getInIndex() const 
	{ 
		return _inIndex; 
	}
	
	//! Ausgangsindex der Leitung.
	/*!
	 * Der Index der Leitung aus Sicht des Vorgaengerknotens.
	 * 
	 * @see setOutIndex
	 * @return Ausgangsindex der Leitung aus Sicht des Vorgaengerknotens.
	 */
	inline unsigned int getOutIndex() const 
	{
		return _outIndex; 
	}
	
	//! Vorgaengerknoten der Leitung.
	/*! 
	 * Der Vorgaengerknoten dieser Leitung.
	 *
	 * @see setFrom
	 * @return Vorgaengerknoten der Leitung.
	 */
	inline Node* From() const 
	{
		return _from; 
	}
	
	//! Nachfolgerknoten der Leitung.
	/*! 
	 * Der Nachfolgerknoten dieser Leitung.
	 *
	 * @see setTo
	 * @return Nachfolgerknoten der Leitung.
	 */
	inline Node* To() const 
	{
		return _to; 
	}


protected:

	//! Eingangsindex der Leitung setzen.
	/*!
	 * Setzt den Index der Leitung aus Sicht des Nachfolgerknotens.
	 * 
	 * @see getInIndex
	 * @param index Index der Leitung aus Sicht des Nachfolgerknoten.
	 */
	inline void setInIndex(unsigned int index)  
	{ 
		_inIndex = index; 
	}
	
	//! Ausgangsindex der Leitung setzen.
	/*!
	 * Setzt den Index der Leitung aus Sicht des Vorgaengerknotens.
	 * 
	 * @see getOutIndex
	 * @param index Index der Leitung bei Vorgaengerknoten.
	 */
	inline void setOutIndex(unsigned int index)  
	{
		_outIndex = index; 
	}

	//! Vorgaengerknoten der Leitung setzen.
	/*!
	 * Setzt den Vorgaengerknoten dieser Leitung.
	 *
	 * @see From
	 * @param from Vorgaengerknoten der Leitung.
	 */
	inline void setFrom(Node* from) 
	{
		_from = from ; 
	}
		
	//! Nachfolgerknoten der Leitung setzen.
	/*!
	 * Setzt den Nachfolgerknoten dieser Leitung.
	 *
	 * @see To
	 * @param to Nachfolgerknoten der Leitung.
	 */
	inline void setTo(Node* to) 
	{
		_to = to; 
	}
	
	//! Implementierung der abstrakten Basisklasse Analysable.
	/*!
	 * Fuer Leitungen sind keine derzeit keine Messungen definiert.
	 *
	 * @see Analysable
	 * @return 0.0
	 */
	double Measure(TypeInfo) 
	{ 
		return 0.0; 
	};


private:
	
	//! Eingangsindex am Router
	unsigned int _inIndex;

	//! Ausgangsindex an einem Router
	unsigned int _outIndex;

	//! Der Startknoten.
	Node* _from;
	
	//! Der Zielknoten.
	Node* _to;

	//! Senderegister eines angeschlossenen Puffers.
	RegisterContainer* _reg;
};

#endif
