#ifndef __WIREMATRIXITERATOR__
#define __WIREMATRIXITERATOR__

#include <wirematrix.h>
#include <wirematrixmodifier.h>
#include <wirematrixentry.h>
#include <wire.h>

//! Abstrakter Zeilen-/Spalteniterator fuer Schaltmatritzen.
/*!
 * Diese Klasse liefert eine abstrakte Spezifikation eines Iterators, der
 * entweder eine Zeile oder eine Spalte einer Schaltmatrix eines Routers 
 * iteriert. Zusammen mit Instanzen der Klasse WireMatrixModifier kann
 * eine Matrix ueber einen Iterator modifiziert werden.
 *
 * @see WireMatrixModifier
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class WireMatrixIterator
{
public:

	//! Aufzaehlung der Iterator-Typen.
	enum e_it_type{ INWIRE_IT, OUTWIRE_IT };
	
	//! Typ des Iterators.
	virtual int rtti() const = 0;
	
	//! Test auf Existenz eines weiteren Eintrages.
	/*!
	 * Diese Methode prueft, ob der Iterator einen Eintrag weiterbewegt
	 * werden kann oder das Ende der Zeile/Spalte erreicht wurde.
	 * 
	 * @return true, falls eine weiterer Matrixeintrag gelesen werden kann.
	 */
	virtual bool hasNext() = 0;
		
	//! Bewegen des Iterators.
	/*!
	 * Diese Methode setzt den Iterator auf den naechsten Eintrag der Matrix.
	 */
	virtual void next() = 0;

	//! Zuruecksetzen des Iterators.
	/*!
	 * Diese Methode setzt den Iterator zurueck auf seine Startposition.
	 */
	virtual void reset() = 0; 

	//! Zugriff auf aktuellen Matrixeintrag.
	/*!
	 * Diese Methode liefert den Eintrag der Matrix zur aktuellen Position
	 * des Iterators.
	 *
	 * @return Aktueller Matrixeintrag.
	 */
	virtual WireMatrixEntry& value() = 0;

	//! Anzahl der iterierten Eintraege in Zeile/Spalte.
	/*!
	 * Diese Methode liefert die Anzahl der Eintraege in der vom Iterator
	 * betrachteten Zeile/Spalte.
	 *
	 * @return Laenge der iterierten Zeile/Spalte.
	 */
	virtual int size() = 0;

	//! Iterator aus Sicht des aktuell iterierten Eintrages.
	/*!
	 * Diese Methode liefert einen Iterator, der diesen Iterator im aktuellen
	 * Eintrag kreuzt, d.h. zu einem Zeileniterator einen Spalteniterator und
	 * umgekehrt.
	 *
	 * @return Kreuzender Iterator zu iteriertem Eintrag.
	 */
	virtual WireMatrixIterator* crossingIterator() = 0;	

	//! Sequentielles Modifizieren eines Eintrages.
	/*!
	 * Diese Methode modifiziert den aktuell iterierten Eintrag der Matrix
	 * durch die Anwendung mehrerer Instanzen der Klasse WireMatrixModifier,
	 * gegeben durch eine Liste. Die Reihenfloge der Anwendung entspricht der
	 * Reihenfolge der Listenelemente.
	 * 
	 * @param list Liste von Instanzen der Klasse WireMatrixModifier.
	 * @return State (TODO)
	 */
	int set(ModifierList* list);

	//! Modifizieren eines Eintrages.
	/*!
	 * Diese Methode modifiziert den aktuell iterierten Eintrag der Matrix
	 * durch die Anwendung der spezifierten Instanz der Klasse WireMatrixModifer.
	 *
	 * @param modifier Instanz der Klasse WireMatrixModifier.
	 * @return Ergebnis der Precondition (WireMatrixModifier::e_precond_return).
	 */
	int set(WireMatrixModifier* modifier);
	
	//! Sequentielles Modifizieren einer Zeile/Spalte.
	/*!
	 * Diese Methode modifiziert die aktuell iterierte Zeile/Spalte der Matrix
	 * durch die Anwendung mehrerer Instanzen der Klasse WireMatrixModifier,
	 * gegeben durch eine Liste, auf jeden einzelnen Eintrag. Die Reihenfloge 
	 * der Anwendung entspricht der Reihenfolge der Listenelemente.
	 * 
	 * @param list Liste von Instanzen der Klasse WireMatrixModifier.
	 * @return State (TODO).
	 */
	int setAll(ModifierList* list);

	//! Modifizieren einer Zeile/Spalte.
	/*!
	 * Diese Methode modifiziert die aktuell iterierte Zeile/Spalte der Matrix
	 * durch die Anwendung der spezifierten Instanz der Klasse WireMatrixModifer.
	 *
	 * @param modifier Instanz der Klasse WireMatrixModifier.
	 * @return State (TODO).
	 */
	int setAll(WireMatrixModifier* modifier);

	virtual ~WireMatrixIterator() {};
};

//! Spalteniterator.
/*!
 * Diese Klasse implementiert die abstrakte Klasse WireMatrixIterator und iteriert die Eintraege
 * einer Spalte, d.h. fuer eine Schaltmatrix eines Routers werden aus Sicht eines Ausgangs alle 
 * Eintraege zu den vorhandenen Eingaengen iteriert.
 *
 * @see WireMatrixIterator
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */

class InWireIterator : public WireMatrixIterator
{
friend class WireMatrix;
	
public:
	
	//! Implementierung der abstrakten Klasse WireMatrixmodifier.
	int rtti() const { return WireMatrixIterator::INWIRE_IT; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	bool hasNext() { return current != it_last; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	void next() { current++; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	void reset() { current = it_first-1; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	WireMatrixEntry& value() { return (current < it_first) ? *it_first : *current; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	int size() { return (*it_last).getInWire()->getInIndex()+1; }
		
	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	WireMatrixIterator* crossingIterator();

	virtual ~InWireIterator() {};
	
protected:
	
	//! Konstruktor.
	/*!
	 * Der Iterator wird von einer Schaltmatrix mit den entsprechenden
	 * Iteratoren einer Matrix bzw. Offsets erstellt.
	 *
	 * @see Matrix
	 * @see WireMatrix
	 * 
	 * @param begin Matrix-Iterator auf Beginn der Spalte.
	 * @param end Matrix-Iterator auf Ende der Spalte.
	 * @param crossing Liste der Zeileniteratoren der Matrix.
	 */
	InWireIterator(	Matrix<WireMatrixEntry>::col_iterator begin,
			Matrix<WireMatrixEntry>::col_iterator end,
			const std::vector<WireMatrixIterator*>& crossing) 
			: it_first(begin), it_last(end-1), current(begin-1), cross_iterators(crossing) {}
			
private:

	//! Interne Repraesentation des Spaltenbeginns.
	Matrix<WireMatrixEntry>::col_iterator it_first;

	//! Interne Repraesentation des Spaltenendes.
	Matrix<WireMatrixEntry>::col_iterator it_last;

	//! Interne Repraesentation der aktuellen Position.
	Matrix<WireMatrixEntry>::col_iterator current;

	//! Zeileniteratoren.
	const std::vector<WireMatrixIterator*>& cross_iterators;
};


//! Zeileniterator
/*!
 * Diese Klasse implementiert die abstrakte Klasse WireMatrixIterator und iteriert die Eintraege
 * einer Zeile, d.h. fuer eine Schaltmatrix eines Routers werden aus Sicht eines Eingangs alle 
 * Eintraege zu den vorhandenen Ausgaengen iteriert.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class OutWireIterator : public WireMatrixIterator
{
friend class WireMatrix;
	
public:

	//! Implementierung der abstrakten Klasse WireMatrixmodifier.
	int rtti() const { return WireMatrixIterator::OUTWIRE_IT; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	bool hasNext() { return current != it_last; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	void next() { current++; }
	
	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	void reset() { current = it_first-1; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	WireMatrixEntry& value() { return (current < it_first) ? *it_first : *current; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	int size() { return (*it_last).getOutWire()->getOutIndex()+1; }

	//! Implementierung der abstrakten Klasse WireMatrixIterator.
	WireMatrixIterator* crossingIterator();

	virtual ~OutWireIterator() {};

protected:
		
	//! Konstruktor.
	/*!
	 * Der Iterator wird von einer Schaltmatrix mit den entsprechenden
	 * Iteratoren einer Matrix bzw. Offsets erstellt.
	 *
	 * @see Matrix
	 * @see WireMatrix
	 * 
	 * @param begin Matrix-Iterator auf Beginn der Zeile.
	 * @param end Matrix-Iterator auf Ende der Zeile.
	 * @param crossing Liste von Spalteniteratoren der Matrix.
	 */
	OutWireIterator(Matrix<WireMatrixEntry>::row_iterator begin,
			Matrix<WireMatrixEntry>::row_iterator end,
			const std::vector<WireMatrixIterator*>& crossing) 
			: it_first(begin), it_last(end-1), current(begin-1), cross_iterators(crossing) {}
			
private:

	//! Interne Repraesentation des Zeilenbeginns.	
	Matrix<WireMatrixEntry>::row_iterator it_first;

	//! Interne Repraesentation des Zeilenendes.
	Matrix<WireMatrixEntry>::row_iterator it_last;

	//! Interne Repraesentation der aktuellen Position in der Zeile.
	Matrix<WireMatrixEntry>::row_iterator current;

	//! Spalteniteratoren.
	const std::vector<WireMatrixIterator*>& cross_iterators;
};


#endif
